#nullable enable
using Canon.Core.Abstractions;
using Canon.Core.GrammarParser;
using Canon.Core.Enums;
namespace Canon.Tests.GeneratedParserTests;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : GrammarParserBase
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "9bb88e74-e446-4ba5-a973-98aa20d700b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "3c7abadd-b010-42b9-8209-178462aeb9fe"}, { new NonTerminator(NonTerminatorType.ProgramHead), "300693b8-1dcc-4005-bdd7-5ffe8998ef9e"}, { new Terminator(KeywordType.Program), "2129199f-ef4f-4744-b1aa-98f15d347179"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9bb88e74-e446-4ba5-a973-98aa20d700b1") },
		{ "3c7abadd-b010-42b9-8209-178462aeb9fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "3c7abadd-b010-42b9-8209-178462aeb9fe") },
		{ "300693b8-1dcc-4005-bdd7-5ffe8998ef9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "fb00e7df-ab46-4cef-89e1-d6c2dc3ae8ea"},}, new Dictionary<Terminator, ReduceInformation>{ }, "300693b8-1dcc-4005-bdd7-5ffe8998ef9e") },
		{ "2129199f-ef4f-4744-b1aa-98f15d347179", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "02139925-d75d-43d9-a84b-e0954ebbb0cc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2129199f-ef4f-4744-b1aa-98f15d347179") },
		{ "fb00e7df-ab46-4cef-89e1-d6c2dc3ae8ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "503e39e9-0727-409b-bc33-5b564c49d77e"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "1597148a-2af8-4387-843d-1ea827599d12"}, { new Terminator(KeywordType.Const), "7764cf75-ac18-40da-9385-ad2c2df34e5f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "fb00e7df-ab46-4cef-89e1-d6c2dc3ae8ea") },
		{ "02139925-d75d-43d9-a84b-e0954ebbb0cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "74000c29-8b55-4903-970f-a2e66c8b4544"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "02139925-d75d-43d9-a84b-e0954ebbb0cc") },
		{ "503e39e9-0727-409b-bc33-5b564c49d77e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "29fe7781-eea5-4ee6-a126-8224f6234b47"},}, new Dictionary<Terminator, ReduceInformation>{ }, "503e39e9-0727-409b-bc33-5b564c49d77e") },
		{ "1597148a-2af8-4387-843d-1ea827599d12", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "eddc09c9-78e3-4990-abc7-23423c52cf46"}, { new Terminator(KeywordType.Var), "3e905ac4-7168-43ea-a793-30f6605a32c8"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "1597148a-2af8-4387-843d-1ea827599d12") },
		{ "7764cf75-ac18-40da-9385-ad2c2df34e5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "e671a1d6-0753-49bd-acb2-a7e5d2f30aae"}, { Terminator.IdentifierTerminator, "2e7aa1ed-2f87-47db-be9a-82cf6d44f9ea"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7764cf75-ac18-40da-9385-ad2c2df34e5f") },
		{ "74000c29-8b55-4903-970f-a2e66c8b4544", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "ea5e800b-5644-4384-a42c-d06fb1003ad3"}, { Terminator.IdentifierTerminator, "fe782bea-42bf-4134-b0e3-dcc36d4d9b0a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "74000c29-8b55-4903-970f-a2e66c8b4544") },
		{ "29fe7781-eea5-4ee6-a126-8224f6234b47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "29fe7781-eea5-4ee6-a126-8224f6234b47") },
		{ "eddc09c9-78e3-4990-abc7-23423c52cf46", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "46857440-56ed-4425-8556-8f06ed9e4934"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "eddc09c9-78e3-4990-abc7-23423c52cf46") },
		{ "3e905ac4-7168-43ea-a793-30f6605a32c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "2695d197-258f-42d2-aa89-035c0c89b32e"}, { new NonTerminator(NonTerminatorType.IdentifierList), "08dd1d92-418a-4be2-9ac5-5734704be8ec"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3e905ac4-7168-43ea-a793-30f6605a32c8") },
		{ "e671a1d6-0753-49bd-acb2-a7e5d2f30aae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "39f83f35-0b83-40db-9ed6-a6ed88e8ed3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e671a1d6-0753-49bd-acb2-a7e5d2f30aae") },
		{ "2e7aa1ed-2f87-47db-be9a-82cf6d44f9ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "38c2a028-7058-418e-8f0a-fcc8c007ac48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2e7aa1ed-2f87-47db-be9a-82cf6d44f9ea") },
		{ "ea5e800b-5644-4384-a42c-d06fb1003ad3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "411f2734-87b0-4486-8293-0a7b2616f90d"}, { new Terminator(DelimiterType.Comma), "784b03ea-488f-4103-8a0b-62273d5370a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ea5e800b-5644-4384-a42c-d06fb1003ad3") },
		{ "fe782bea-42bf-4134-b0e3-dcc36d4d9b0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "fe782bea-42bf-4134-b0e3-dcc36d4d9b0a") },
		{ "46857440-56ed-4425-8556-8f06ed9e4934", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "35baa6d3-26bb-482d-a826-442940dae2ab"}, { new Terminator(KeywordType.Begin), "f1101569-f974-4fcc-b9c3-67c719532345"}, { new NonTerminator(NonTerminatorType.Subprogram), "d6265cb1-2de6-463b-90dc-c0dba50a421d"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "d9f5cd21-2975-43a3-a072-ea3222d11351"}, { new Terminator(KeywordType.Procedure), "0f4164ef-7dd8-40d5-b905-b46e12504a2d"}, { new Terminator(KeywordType.Function), "f2950758-d9d4-4a2d-b2dd-91b69bf38331"},}, new Dictionary<Terminator, ReduceInformation>{ }, "46857440-56ed-4425-8556-8f06ed9e4934") },
		{ "2695d197-258f-42d2-aa89-035c0c89b32e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "21810ffe-c1cb-4cfa-98e6-4091b76ceae2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2695d197-258f-42d2-aa89-035c0c89b32e") },
		{ "08dd1d92-418a-4be2-9ac5-5734704be8ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "2e9512a6-1f96-4031-87e8-29e53788045d"}, { new Terminator(DelimiterType.Comma), "11c0852f-9d83-4729-b039-69e05b9e04df"},}, new Dictionary<Terminator, ReduceInformation>{ }, "08dd1d92-418a-4be2-9ac5-5734704be8ec") },
		{ "280edb8d-540f-42a6-be37-39effb326c4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "280edb8d-540f-42a6-be37-39effb326c4e") },
		{ "39f83f35-0b83-40db-9ed6-a6ed88e8ed3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "31aa280d-8deb-4f4c-85b3-370a3928508b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "39f83f35-0b83-40db-9ed6-a6ed88e8ed3a") },
		{ "38c2a028-7058-418e-8f0a-fcc8c007ac48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "7b8ae1e2-8a79-472a-b94b-037d5fbcaaad"}, { new Terminator(OperatorType.Plus), "59e3cc3e-5c7a-4e46-8dac-11fb11f27224"}, { new Terminator(OperatorType.Minus), "ac30e077-bb51-4cc3-8134-545edafd4cab"}, { Terminator.NumberTerminator, "cd35c768-a64a-426f-a1b6-e33cf01517b8"}, { Terminator.CharacterTerminator, "5abd0baa-cb16-4ed6-93a1-6abf8347e474"},}, new Dictionary<Terminator, ReduceInformation>{ }, "38c2a028-7058-418e-8f0a-fcc8c007ac48") },
		{ "411f2734-87b0-4486-8293-0a7b2616f90d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "411f2734-87b0-4486-8293-0a7b2616f90d") },
		{ "784b03ea-488f-4103-8a0b-62273d5370a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "73163756-45cb-47bc-8072-e3b5e1a9d6f3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "784b03ea-488f-4103-8a0b-62273d5370a6") },
		{ "35baa6d3-26bb-482d-a826-442940dae2ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "35baa6d3-26bb-482d-a826-442940dae2ab") },
		{ "f1101569-f974-4fcc-b9c3-67c719532345", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "f97e0c9e-c17e-4a7d-afbc-4330356b5557"}, { new NonTerminator(NonTerminatorType.Statement), "669b7b94-f3e0-493a-8de2-511a254a494b"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "f1101569-f974-4fcc-b9c3-67c719532345") },
		{ "d6265cb1-2de6-463b-90dc-c0dba50a421d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "54a4d440-9aae-4115-89dd-f6578ed5de3b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6265cb1-2de6-463b-90dc-c0dba50a421d") },
		{ "d9f5cd21-2975-43a3-a072-ea3222d11351", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "f9c6721f-a109-43fa-99ee-517e85a2fbd0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d9f5cd21-2975-43a3-a072-ea3222d11351") },
		{ "0f4164ef-7dd8-40d5-b905-b46e12504a2d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "9aa0297b-850c-450c-914c-42637d60a407"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0f4164ef-7dd8-40d5-b905-b46e12504a2d") },
		{ "f2950758-d9d4-4a2d-b2dd-91b69bf38331", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "a081a6f9-5bd2-48d3-8bfd-a2192fe07852"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f2950758-d9d4-4a2d-b2dd-91b69bf38331") },
		{ "21810ffe-c1cb-4cfa-98e6-4091b76ceae2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "f7b5b670-d255-4808-9734-fdc4b42badb3"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "21810ffe-c1cb-4cfa-98e6-4091b76ceae2") },
		{ "2e9512a6-1f96-4031-87e8-29e53788045d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "400ce418-4452-40cb-86c3-4f9cf4329bb1"}, { new NonTerminator(NonTerminatorType.BasicType), "c6c88ade-10e5-419c-b341-8e73ca00d79b"}, { new Terminator(KeywordType.Array), "99ab4307-9a24-43df-9601-847e9dd27ddc"}, { new Terminator(KeywordType.Integer), "a96775ea-398e-416c-803c-842482059a00"}, { new Terminator(KeywordType.Real), "815f5e68-35bd-4247-966e-8b3824c407dd"}, { new Terminator(KeywordType.Boolean), "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0"}, { new Terminator(KeywordType.Character), "90564982-90b1-4eaf-a3c5-467d173f730a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2e9512a6-1f96-4031-87e8-29e53788045d") },
		{ "11c0852f-9d83-4729-b039-69e05b9e04df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "f33257c7-7504-44d3-b908-2607c9b5326b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "11c0852f-9d83-4729-b039-69e05b9e04df") },
		{ "31aa280d-8deb-4f4c-85b3-370a3928508b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "4fe022d0-d891-49f2-858d-7314bbc6ecf5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31aa280d-8deb-4f4c-85b3-370a3928508b") },
		{ "7b8ae1e2-8a79-472a-b94b-037d5fbcaaad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "7b8ae1e2-8a79-472a-b94b-037d5fbcaaad") },
		{ "59e3cc3e-5c7a-4e46-8dac-11fb11f27224", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "897ff0ab-640e-4c94-a95a-dd1ef233d299"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59e3cc3e-5c7a-4e46-8dac-11fb11f27224") },
		{ "ac30e077-bb51-4cc3-8134-545edafd4cab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "d21ecc85-1fb1-4e73-9837-b92f5887b30c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac30e077-bb51-4cc3-8134-545edafd4cab") },
		{ "cd35c768-a64a-426f-a1b6-e33cf01517b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "cd35c768-a64a-426f-a1b6-e33cf01517b8") },
		{ "5abd0baa-cb16-4ed6-93a1-6abf8347e474", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "5abd0baa-cb16-4ed6-93a1-6abf8347e474") },
		{ "73163756-45cb-47bc-8072-e3b5e1a9d6f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "73163756-45cb-47bc-8072-e3b5e1a9d6f3") },
		{ "f97e0c9e-c17e-4a7d-afbc-4330356b5557", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "8afa13cc-f5f2-4187-adc4-2527fb79f2b2"}, { new Terminator(DelimiterType.Semicolon), "56452484-c5ea-4e76-9fdf-c2dbc7763e55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f97e0c9e-c17e-4a7d-afbc-4330356b5557") },
		{ "669b7b94-f3e0-493a-8de2-511a254a494b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "669b7b94-f3e0-493a-8de2-511a254a494b") },
		{ "340c5627-5962-46b7-982e-a7f4da6071ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "6c490044-8b40-4b3e-abd2-9eaff3ef5aee"},}, new Dictionary<Terminator, ReduceInformation>{ }, "340c5627-5962-46b7-982e-a7f4da6071ef") },
		{ "15a67b60-7589-4ccd-b0a1-e0e05e419a76", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "9cf00111-9214-4533-8a5d-534ae89abb84"}, { new NonTerminator(NonTerminatorType.IdVarPart), "2b607612-2ceb-45fd-b56a-e86a62d2f563"}, { new Terminator(DelimiterType.LeftSquareBracket), "4160990e-129b-441e-9e5b-4a514c45faef"}, { new Terminator(DelimiterType.LeftParenthesis), "b56f8824-9054-4c0b-a8ea-a47ded6764c6"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "15a67b60-7589-4ccd-b0a1-e0e05e419a76") },
		{ "6f286d93-70c5-447e-96f3-55dd578788bc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "6f286d93-70c5-447e-96f3-55dd578788bc") },
		{ "baa01d82-c80e-429e-948f-e1c69ad8c4c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "baa01d82-c80e-429e-948f-e1c69ad8c4c4") },
		{ "b43df555-7769-42ab-b698-430d230f6a3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c76d3ff4-8853-4921-9857-a81b2049ab67"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9015fb18-d537-480b-851b-9ddc84c18490"}, { new NonTerminator(NonTerminatorType.Term), "c428bc45-4c85-4c95-8c80-925e92f7c52e"}, { new NonTerminator(NonTerminatorType.Factor), "a9eca599-2434-41b1-93f5-2f6966bc3580"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b43df555-7769-42ab-b698-430d230f6a3e") },
		{ "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "b2c0a29b-8024-4176-939c-e6ff87362633"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f") },
		{ "6e50af1f-a22a-432f-80cd-9c938375d882", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "e644cc50-e97e-44f8-b9e9-309448281f96"}, { new NonTerminator(NonTerminatorType.Statement), "669b7b94-f3e0-493a-8de2-511a254a494b"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "6e50af1f-a22a-432f-80cd-9c938375d882") },
		{ "54a4d440-9aae-4115-89dd-f6578ed5de3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "54a4d440-9aae-4115-89dd-f6578ed5de3b") },
		{ "f9c6721f-a109-43fa-99ee-517e85a2fbd0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "a6fab468-6110-4c4c-90c3-1798bb3cc310"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "335cb0ab-fd8a-4ff5-9847-23583bfce8ab"}, { new Terminator(KeywordType.Const), "7764cf75-ac18-40da-9385-ad2c2df34e5f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "f9c6721f-a109-43fa-99ee-517e85a2fbd0") },
		{ "9aa0297b-850c-450c-914c-42637d60a407", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "9125752d-af86-430d-942f-5daeaff090f2"}, { new Terminator(DelimiterType.LeftParenthesis), "1737c74d-47f1-45a7-adb8-c0c6e887904e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "9aa0297b-850c-450c-914c-42637d60a407") },
		{ "a081a6f9-5bd2-48d3-8bfd-a2192fe07852", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "b70e1b50-8cb6-4f4a-9d1e-04b29f9f758c"}, { new Terminator(DelimiterType.LeftParenthesis), "a8abfeba-98a7-4280-9703-690d347b90c0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "a081a6f9-5bd2-48d3-8bfd-a2192fe07852") },
		{ "f7b5b670-d255-4808-9734-fdc4b42badb3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "48d6d6c3-306e-4a33-93db-39aedcb501ab"}, { new Terminator(DelimiterType.Comma), "11c0852f-9d83-4729-b039-69e05b9e04df"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f7b5b670-d255-4808-9734-fdc4b42badb3") },
		{ "400ce418-4452-40cb-86c3-4f9cf4329bb1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "400ce418-4452-40cb-86c3-4f9cf4329bb1") },
		{ "c6c88ade-10e5-419c-b341-8e73ca00d79b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "c6c88ade-10e5-419c-b341-8e73ca00d79b") },
		{ "99ab4307-9a24-43df-9601-847e9dd27ddc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "ca3a8c09-c0e5-4fa1-8f90-856d6c23cda2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "99ab4307-9a24-43df-9601-847e9dd27ddc") },
		{ "a96775ea-398e-416c-803c-842482059a00", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "a96775ea-398e-416c-803c-842482059a00") },
		{ "815f5e68-35bd-4247-966e-8b3824c407dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "815f5e68-35bd-4247-966e-8b3824c407dd") },
		{ "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0") },
		{ "90564982-90b1-4eaf-a3c5-467d173f730a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "90564982-90b1-4eaf-a3c5-467d173f730a") },
		{ "f33257c7-7504-44d3-b908-2607c9b5326b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "f33257c7-7504-44d3-b908-2607c9b5326b") },
		{ "4fe022d0-d891-49f2-858d-7314bbc6ecf5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "42ecb3b5-4cf2-448d-ab4f-c52f9b50341c"}, { new Terminator(OperatorType.Plus), "59e3cc3e-5c7a-4e46-8dac-11fb11f27224"}, { new Terminator(OperatorType.Minus), "ac30e077-bb51-4cc3-8134-545edafd4cab"}, { Terminator.NumberTerminator, "cd35c768-a64a-426f-a1b6-e33cf01517b8"}, { Terminator.CharacterTerminator, "5abd0baa-cb16-4ed6-93a1-6abf8347e474"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4fe022d0-d891-49f2-858d-7314bbc6ecf5") },
		{ "897ff0ab-640e-4c94-a95a-dd1ef233d299", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "897ff0ab-640e-4c94-a95a-dd1ef233d299") },
		{ "d21ecc85-1fb1-4e73-9837-b92f5887b30c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "d21ecc85-1fb1-4e73-9837-b92f5887b30c") },
		{ "8afa13cc-f5f2-4187-adc4-2527fb79f2b2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "8afa13cc-f5f2-4187-adc4-2527fb79f2b2") },
		{ "56452484-c5ea-4e76-9fdf-c2dbc7763e55", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "03b0fb6c-87e3-4daf-9d1a-8bba01fa7bae"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "56452484-c5ea-4e76-9fdf-c2dbc7763e55") },
		{ "6c490044-8b40-4b3e-abd2-9eaff3ef5aee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "94d5fe93-f6ba-4eee-aeba-f422e629fc1d"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "7a5a0791-22d4-4d17-8050-b2f2b1c1e30c"}, { new NonTerminator(NonTerminatorType.Term), "9f109ab6-c2a0-4976-b465-647e31fb1f0d"}, { new NonTerminator(NonTerminatorType.Factor), "79707115-1161-4189-b15c-42be5f0e7184"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6c490044-8b40-4b3e-abd2-9eaff3ef5aee") },
		{ "9cf00111-9214-4533-8a5d-534ae89abb84", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "0b5eae12-693d-4221-952a-cf5a862ec0c9"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "7a5a0791-22d4-4d17-8050-b2f2b1c1e30c"}, { new NonTerminator(NonTerminatorType.Term), "9f109ab6-c2a0-4976-b465-647e31fb1f0d"}, { new NonTerminator(NonTerminatorType.Factor), "79707115-1161-4189-b15c-42be5f0e7184"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9cf00111-9214-4533-8a5d-534ae89abb84") },
		{ "2b607612-2ceb-45fd-b56a-e86a62d2f563", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "2b607612-2ceb-45fd-b56a-e86a62d2f563") },
		{ "4160990e-129b-441e-9e5b-4a514c45faef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "7b8f5c82-0d4f-4d59-8d3c-d2ffa030ba65"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4160990e-129b-441e-9e5b-4a514c45faef") },
		{ "b56f8824-9054-4c0b-a8ea-a47ded6764c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f7c3cda3-3b1b-4acb-b973-c5ccb4b028ee"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b56f8824-9054-4c0b-a8ea-a47ded6764c6") },
		{ "c76d3ff4-8853-4921-9857-a81b2049ab67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "a20e8923-a22c-4ee4-b450-b8cda9b9fb62"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c76d3ff4-8853-4921-9857-a81b2049ab67") },
		{ "9015fb18-d537-480b-851b-9ddc84c18490", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "2c707e23-e9fc-421b-a79a-12f5a07675c1"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "cfa3723e-fd1a-4b34-aa85-692eb33d1409"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "9015fb18-d537-480b-851b-9ddc84c18490") },
		{ "c428bc45-4c85-4c95-8c80-925e92f7c52e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "122ff222-3ee7-48f0-afe3-e9ae0f6b0102"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c428bc45-4c85-4c95-8c80-925e92f7c52e") },
		{ "a9eca599-2434-41b1-93f5-2f6966bc3580", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a9eca599-2434-41b1-93f5-2f6966bc3580") },
		{ "ae79fdb0-d398-4ac7-b604-069a7d67076a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "ae79fdb0-d398-4ac7-b604-069a7d67076a") },
		{ "50dd712b-3a18-4419-9af4-91e2b902a793", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "50dd712b-3a18-4419-9af4-91e2b902a793") },
		{ "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "62042602-3a08-46ec-bfd1-11b6aa253fdb"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa") },
		{ "546e5c99-a3fb-4bcb-98ac-01aacabb3d51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "dc4ef25e-49ce-45d1-9656-e3c917a70859"}, { new NonTerminator(NonTerminatorType.IdVarPart), "0110a1aa-0262-4422-8e2b-d7a01642a90f"}, { new Terminator(DelimiterType.LeftSquareBracket), "91776258-6aa6-414d-a5e5-9d07eaf906f4"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51") },
		{ "eab617f6-a74a-4e0e-86a6-6ab31735be87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "395af803-a00f-4008-8516-3be83f25ddc9"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eab617f6-a74a-4e0e-86a6-6ab31735be87") },
		{ "8adceea0-905c-42b1-879f-562bbb8bcd94", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9202d9c8-7ac2-4a24-a69d-c6b25e54e2ef"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8adceea0-905c-42b1-879f-562bbb8bcd94") },
		{ "b2c0a29b-8024-4176-939c-e6ff87362633", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "3571df43-74df-4d43-adcd-1d33d3dee369"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b2c0a29b-8024-4176-939c-e6ff87362633") },
		{ "e644cc50-e97e-44f8-b9e9-309448281f96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "e1258288-dbbf-4a4b-8053-f2c00e28b5a2"}, { new Terminator(DelimiterType.Semicolon), "56452484-c5ea-4e76-9fdf-c2dbc7763e55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e644cc50-e97e-44f8-b9e9-309448281f96") },
		{ "a6fab468-6110-4c4c-90c3-1798bb3cc310", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "a6fab468-6110-4c4c-90c3-1798bb3cc310") },
		{ "335cb0ab-fd8a-4ff5-9847-23583bfce8ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "c1f4db78-2a94-47c7-8ca7-10ee1672c648"}, { new Terminator(KeywordType.Var), "3e905ac4-7168-43ea-a793-30f6605a32c8"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "335cb0ab-fd8a-4ff5-9847-23583bfce8ab") },
		{ "9125752d-af86-430d-942f-5daeaff090f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "9125752d-af86-430d-942f-5daeaff090f2") },
		{ "1737c74d-47f1-45a7-adb8-c0c6e887904e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "644ef13e-ba59-47a5-8e94-b711f45860fd"}, { new NonTerminator(NonTerminatorType.Parameter), "5c5556a0-f28d-4d7b-af4f-566e5f812929"}, { new NonTerminator(NonTerminatorType.VarParameter), "60ba72c9-f75f-4c97-b688-5541f9960d6f"}, { new NonTerminator(NonTerminatorType.ValueParameter), "1528bf29-e941-4a4c-ad56-962bf9bec1a2"}, { new Terminator(KeywordType.Var), "766b438c-646b-41c1-8dee-9fc0ff7b37d5"}, { new NonTerminator(NonTerminatorType.IdentifierList), "56e8abe4-943b-47d2-80f7-181a6bae7fad"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1737c74d-47f1-45a7-adb8-c0c6e887904e") },
		{ "b70e1b50-8cb6-4f4a-9d1e-04b29f9f758c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "de0d1ef0-edc6-45a7-81a7-9e48f7d4cb20"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b70e1b50-8cb6-4f4a-9d1e-04b29f9f758c") },
		{ "a8abfeba-98a7-4280-9703-690d347b90c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ParameterList), "5f14fe2b-b48c-485f-9cc2-b60c8e5db053"}, { new NonTerminator(NonTerminatorType.Parameter), "5c5556a0-f28d-4d7b-af4f-566e5f812929"}, { new NonTerminator(NonTerminatorType.VarParameter), "60ba72c9-f75f-4c97-b688-5541f9960d6f"}, { new NonTerminator(NonTerminatorType.ValueParameter), "1528bf29-e941-4a4c-ad56-962bf9bec1a2"}, { new Terminator(KeywordType.Var), "766b438c-646b-41c1-8dee-9fc0ff7b37d5"}, { new NonTerminator(NonTerminatorType.IdentifierList), "56e8abe4-943b-47d2-80f7-181a6bae7fad"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a8abfeba-98a7-4280-9703-690d347b90c0") },
		{ "48d6d6c3-306e-4a33-93db-39aedcb501ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "4812c3bb-21e1-4cc1-93e8-bfbcc92e6142"}, { new NonTerminator(NonTerminatorType.BasicType), "c6c88ade-10e5-419c-b341-8e73ca00d79b"}, { new Terminator(KeywordType.Array), "99ab4307-9a24-43df-9601-847e9dd27ddc"}, { new Terminator(KeywordType.Integer), "a96775ea-398e-416c-803c-842482059a00"}, { new Terminator(KeywordType.Real), "815f5e68-35bd-4247-966e-8b3824c407dd"}, { new Terminator(KeywordType.Boolean), "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0"}, { new Terminator(KeywordType.Character), "90564982-90b1-4eaf-a3c5-467d173f730a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "48d6d6c3-306e-4a33-93db-39aedcb501ab") },
		{ "ca3a8c09-c0e5-4fa1-8f90-856d6c23cda2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "539de0b8-8967-4976-b099-82693968ec05"}, { Terminator.NumberTerminator, "aab716cb-df44-4886-9051-967c381a29e9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ca3a8c09-c0e5-4fa1-8f90-856d6c23cda2") },
		{ "42ecb3b5-4cf2-448d-ab4f-c52f9b50341c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "42ecb3b5-4cf2-448d-ab4f-c52f9b50341c") },
		{ "03b0fb6c-87e3-4daf-9d1a-8bba01fa7bae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "03b0fb6c-87e3-4daf-9d1a-8bba01fa7bae") },
		{ "94d5fe93-f6ba-4eee-aeba-f422e629fc1d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "94d5fe93-f6ba-4eee-aeba-f422e629fc1d") },
		{ "7a5a0791-22d4-4d17-8050-b2f2b1c1e30c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "11a44452-1d7d-484f-a590-318fc272a984"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "83089975-522a-4cc8-8043-15a6e7df9db6"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "7a5a0791-22d4-4d17-8050-b2f2b1c1e30c") },
		{ "9f109ab6-c2a0-4976-b465-647e31fb1f0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "30390509-2485-4027-b468-7f5a06f45911"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9f109ab6-c2a0-4976-b465-647e31fb1f0d") },
		{ "79707115-1161-4189-b15c-42be5f0e7184", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "79707115-1161-4189-b15c-42be5f0e7184") },
		{ "b0da74db-1321-4ad9-9e15-008c14315e6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b0da74db-1321-4ad9-9e15-008c14315e6d") },
		{ "084e5d87-68de-499f-9563-726dfa760000", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "084e5d87-68de-499f-9563-726dfa760000") },
		{ "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "86010c6f-fdbd-4019-8d71-02ede82abc22"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60") },
		{ "bb7a33ac-07b6-4927-8505-fb4f90091683", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "7606f153-120c-4355-ae8a-59dec9fc2258"}, { new NonTerminator(NonTerminatorType.IdVarPart), "78ecf1b9-59f7-46d4-aceb-a8fd852a9230"}, { new Terminator(DelimiterType.LeftSquareBracket), "80664dd8-50f0-4861-9f93-1f4772365f4b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "bb7a33ac-07b6-4927-8505-fb4f90091683") },
		{ "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "85199747-3d53-403c-9e46-62ca7127d2ef"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482") },
		{ "f1c47f00-204c-43e5-bfe6-6877e4b3ef48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "84cdc13a-c47c-4a6c-8da5-bcab90220fee"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f1c47f00-204c-43e5-bfe6-6877e4b3ef48") },
		{ "0b5eae12-693d-4221-952a-cf5a862ec0c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "0b5eae12-693d-4221-952a-cf5a862ec0c9") },
		{ "7b8f5c82-0d4f-4d59-8d3c-d2ffa030ba65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "67a0b1df-6860-454e-a4d5-a0f5a53107c6"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7b8f5c82-0d4f-4d59-8d3c-d2ffa030ba65") },
		{ "34aaf0d9-64b0-4678-a10c-a1e4e664fce6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "34aaf0d9-64b0-4678-a10c-a1e4e664fce6") },
		{ "1348560a-2744-469d-87a6-092c03873ff0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "14e906d4-b447-487c-a383-726a031a8886"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "b10ec06e-38db-459f-bdf5-d8073ce98bf3"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "1348560a-2744-469d-87a6-092c03873ff0") },
		{ "154bd632-5bfd-4450-9525-771afbbdf5dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "097a5289-eae6-4e48-989f-82cd5230ec8f"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "154bd632-5bfd-4450-9525-771afbbdf5dd") },
		{ "4b7672cc-8831-48ba-b603-082dcc6c0aec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "4b7672cc-8831-48ba-b603-082dcc6c0aec") },
		{ "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7") },
		{ "fafa494b-d3c6-4502-aef9-ad41dc354967", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fafa494b-d3c6-4502-aef9-ad41dc354967") },
		{ "a4490596-3b7b-4793-b558-c732bcced8bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f50bc9a0-9e95-4ac1-905e-663ec0496cf2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a4490596-3b7b-4793-b558-c732bcced8bd") },
		{ "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "93476f7b-7fdf-4e7a-8d96-dfe48640e626"}, { new NonTerminator(NonTerminatorType.IdVarPart), "130b533f-8e3f-4449-99b7-a5fe44f6df4c"}, { new Terminator(DelimiterType.LeftSquareBracket), "e8d686bd-33cb-48f4-86a5-6fa0e51874ef"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708") },
		{ "8c87e7ad-3592-46b9-8fa5-78261721f178", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "055269e0-0802-46b2-aad9-99fa3cae44ac"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8c87e7ad-3592-46b9-8fa5-78261721f178") },
		{ "5eaad4b8-8722-4df1-bd50-173292b1abad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7ff693b4-56b3-4bbf-be07-a487463a04bd"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5eaad4b8-8722-4df1-bd50-173292b1abad") },
		{ "f7c3cda3-3b1b-4acb-b973-c5ccb4b028ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "96f4075f-ecea-41f2-9962-ca38daaf80e6"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f7c3cda3-3b1b-4acb-b973-c5ccb4b028ee") },
		{ "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f") },
		{ "faf7f4cb-301e-45d9-9745-b6d82ce096b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "2c643f3e-c602-4575-b2c4-e44565fc735e"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "be234840-b527-4931-b33c-d59e92d9c04c"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "faf7f4cb-301e-45d9-9745-b6d82ce096b8") },
		{ "d87cf8df-734d-45bd-b403-027d25cee4b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c0662fa7-53cc-443b-869c-32aaf59fa3fe"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d87cf8df-734d-45bd-b403-027d25cee4b4") },
		{ "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4") },
		{ "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea") },
		{ "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5") },
		{ "43def0a3-5d0a-4fe6-ac26-aeff86ab398a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1f5ad0f3-9700-4202-9347-6d5b90058933"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "43def0a3-5d0a-4fe6-ac26-aeff86ab398a") },
		{ "e9da9819-f2cf-4a67-9513-48874b3bbce2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "452e8264-72d7-4819-bb61-d49c33976c1f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d1eb42cd-85e2-4165-a008-b68c8a312337"}, { new Terminator(DelimiterType.LeftSquareBracket), "d068a8bc-d79e-41fd-946b-e45864d58a1b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e9da9819-f2cf-4a67-9513-48874b3bbce2") },
		{ "1af55271-632f-4aa4-b4ef-31530ba39a80", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "018b5b46-29d9-475c-b426-c6317b50dd8c"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1af55271-632f-4aa4-b4ef-31530ba39a80") },
		{ "b5b15cf7-d149-4a37-8ac2-619e23d6076b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0190cdbf-ca19-4e97-b786-5c121f2da4a8"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b5b15cf7-d149-4a37-8ac2-619e23d6076b") },
		{ "a20e8923-a22c-4ee4-b450-b8cda9b9fb62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "7028e224-1791-4dcb-b0c5-5aab5913c4ee"}, { new NonTerminator(NonTerminatorType.Variable), "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36"}, { Terminator.IdentifierTerminator, "d74ee885-45f2-4595-a66a-53e932d8391f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b8794192-f4c9-4a1b-916f-bfaf49f43208"}, { new Terminator(KeywordType.If), "f6810f84-1a46-4894-8c44-c2f8068e5c2f"}, { new Terminator(KeywordType.For), "954e1e88-c806-45af-84bf-a00026c87a9c"}, { new Terminator(KeywordType.Begin), "52ecab85-fa9a-45b6-942d-b6de60d7087c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "a20e8923-a22c-4ee4-b450-b8cda9b9fb62") },
		{ "2c707e23-e9fc-421b-a79a-12f5a07675c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "98c75aa1-83a7-4ef5-ab87-78fddde6d44e"}, { new NonTerminator(NonTerminatorType.Term), "a2432249-8085-4c99-9c29-761396d84e3d"}, { new NonTerminator(NonTerminatorType.Factor), "428e3613-c455-41cc-af27-0a1c2559081c"}, { Terminator.NumberTerminator, "1a7adbd9-96de-48b1-825b-50f70daf617b"}, { new NonTerminator(NonTerminatorType.Variable), "b53dfb08-0712-473a-9b0d-55eea1a5a3d1"}, { new Terminator(DelimiterType.LeftParenthesis), "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b"}, { Terminator.IdentifierTerminator, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4"}, { new Terminator(KeywordType.Not), "e074f32e-891a-483b-bafe-b284c053fe26"}, { new Terminator(OperatorType.Minus), "7f436b2a-1c49-4827-abce-bf45864feb99"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2c707e23-e9fc-421b-a79a-12f5a07675c1") },
		{ "0e7d5c37-773d-432f-a8cf-784bb6b0efe3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "0e7d5c37-773d-432f-a8cf-784bb6b0efe3") },
		{ "be609488-c42f-47f8-a1fd-f2d444c93bc8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "be609488-c42f-47f8-a1fd-f2d444c93bc8") },
		{ "44f4764c-0b4f-4c28-864a-1001907823d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "44f4764c-0b4f-4c28-864a-1001907823d5") },
		{ "07064b0f-4530-4a28-8fa2-42b3fdb533b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "07064b0f-4530-4a28-8fa2-42b3fdb533b1") },
		{ "75cf3ae1-b17b-4a65-a659-706f30e30068", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "75cf3ae1-b17b-4a65-a659-706f30e30068") },
		{ "177eed3f-ff83-48f0-88e3-46d396939896", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "177eed3f-ff83-48f0-88e3-46d396939896") },
		{ "cfa3723e-fd1a-4b34-aa85-692eb33d1409", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b03c1c3d-d00e-4d4a-922c-4eca64816f52"}, { new NonTerminator(NonTerminatorType.Factor), "a9eca599-2434-41b1-93f5-2f6966bc3580"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cfa3723e-fd1a-4b34-aa85-692eb33d1409") },
		{ "bee54548-de25-4e96-8c73-9e234b72bc44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "bee54548-de25-4e96-8c73-9e234b72bc44") },
		{ "d94f09ba-e20f-4b58-8e43-bad97eb903f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "d94f09ba-e20f-4b58-8e43-bad97eb903f4") },
		{ "65027cc5-09b2-4c80-b933-3747f52730e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "65027cc5-09b2-4c80-b933-3747f52730e9") },
		{ "122ff222-3ee7-48f0-afe3-e9ae0f6b0102", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4640daab-e55c-4851-882c-e1e2b04689d3"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "122ff222-3ee7-48f0-afe3-e9ae0f6b0102") },
		{ "e0f8ce47-09b6-4bd5-86c7-e63662ad872b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "e0f8ce47-09b6-4bd5-86c7-e63662ad872b") },
		{ "00b1e30f-225b-4646-9de1-01d6655c252d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "00b1e30f-225b-4646-9de1-01d6655c252d") },
		{ "b75ade6d-fec8-4af0-a07c-4299e12e924f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "b75ade6d-fec8-4af0-a07c-4299e12e924f") },
		{ "e5f4e1f6-05cd-462c-8b33-5270c025dccd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "e5f4e1f6-05cd-462c-8b33-5270c025dccd") },
		{ "942890b9-2bb6-47b5-8481-9efab246471d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "942890b9-2bb6-47b5-8481-9efab246471d") },
		{ "62042602-3a08-46ec-bfd1-11b6aa253fdb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "395d0cb9-d970-434b-aeee-fb8d7e471598"},}, new Dictionary<Terminator, ReduceInformation>{ }, "62042602-3a08-46ec-bfd1-11b6aa253fdb") },
		{ "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "dedcb900-f845-408f-a097-1a62c07b2b38"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "aaa50ef2-c576-441a-8a35-dfa53e223083"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d") },
		{ "c4cfbf2e-bfc7-4fa0-9534-3832086049a4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4cc50272-72e3-4af7-b89d-8b06e7dddf37"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c4cfbf2e-bfc7-4fa0-9534-3832086049a4") },
		{ "15f7f03e-89b0-4b94-b82c-f780ab6d2668", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "15f7f03e-89b0-4b94-b82c-f780ab6d2668") },
		{ "1e55cb78-8ebd-47b0-af5e-601c90cb3927", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1e55cb78-8ebd-47b0-af5e-601c90cb3927") },
		{ "aff1b54e-2140-48a5-ac62-65651706cd17", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "aff1b54e-2140-48a5-ac62-65651706cd17") },
		{ "fe073a91-3867-45bb-b17b-793e15e3f602", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "a7237bd4-1e47-42a4-ace0-1e07ed47c5a3"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fe073a91-3867-45bb-b17b-793e15e3f602") },
		{ "5a5ea736-2683-45c7-b635-07630a0dfd22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "4baa2207-1c00-4b57-884d-227000c3348e"}, { new NonTerminator(NonTerminatorType.IdVarPart), "488a8f93-38d3-4aae-a518-9898727871a1"}, { new Terminator(DelimiterType.LeftSquareBracket), "e68ab851-7f93-41e2-a07b-e069cf268c0a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5a5ea736-2683-45c7-b635-07630a0dfd22") },
		{ "b409d27b-d192-4a8e-bae2-7545bdfc8a65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "cbbbcb62-bac0-431a-95ba-e175acf5d159"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b409d27b-d192-4a8e-bae2-7545bdfc8a65") },
		{ "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b9fd39ed-e892-4c6d-a3a4-d82899ce6f3b"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec") },
		{ "dc4ef25e-49ce-45d1-9656-e3c917a70859", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "8a1c2190-7063-4afa-b7a4-d8e875d65598"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc4ef25e-49ce-45d1-9656-e3c917a70859") },
		{ "0110a1aa-0262-4422-8e2b-d7a01642a90f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "0110a1aa-0262-4422-8e2b-d7a01642a90f") },
		{ "91776258-6aa6-414d-a5e5-9d07eaf906f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "5b9df392-de08-4a85-80a6-9112c10a671f"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "91776258-6aa6-414d-a5e5-9d07eaf906f4") },
		{ "395af803-a00f-4008-8516-3be83f25ddc9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "395af803-a00f-4008-8516-3be83f25ddc9") },
		{ "9202d9c8-7ac2-4a24-a69d-c6b25e54e2ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9202d9c8-7ac2-4a24-a69d-c6b25e54e2ef") },
		{ "3571df43-74df-4d43-adcd-1d33d3dee369", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "e680f46b-abe5-483e-823e-ba060964f9fc"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9c417c34-245c-4d71-ac9b-a095bc19f366"}, { new NonTerminator(NonTerminatorType.Term), "9b515565-da09-46ce-aa9d-2a7969ad537c"}, { new NonTerminator(NonTerminatorType.Factor), "ae5fa63b-fa83-4af1-a392-77c6c28fc8c9"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3571df43-74df-4d43-adcd-1d33d3dee369") },
		{ "e1258288-dbbf-4a4b-8053-f2c00e28b5a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "e1258288-dbbf-4a4b-8053-f2c00e28b5a2") },
		{ "c1f4db78-2a94-47c7-8ca7-10ee1672c648", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "a80d1e17-6b63-488b-9d67-f75ec27b859d"}, { new Terminator(KeywordType.Begin), "54b3ebdc-53af-4829-9628-3af934c9bfa6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c1f4db78-2a94-47c7-8ca7-10ee1672c648") },
		{ "644ef13e-ba59-47a5-8e94-b711f45860fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4b931270-e39b-4d43-8cff-da0fa97ae435"}, { new Terminator(DelimiterType.Semicolon), "8e49eb1f-4a76-463e-b84f-dcba0c4789fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "644ef13e-ba59-47a5-8e94-b711f45860fd") },
		{ "5c5556a0-f28d-4d7b-af4f-566e5f812929", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "5c5556a0-f28d-4d7b-af4f-566e5f812929") },
		{ "60ba72c9-f75f-4c97-b688-5541f9960d6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "60ba72c9-f75f-4c97-b688-5541f9960d6f") },
		{ "1528bf29-e941-4a4c-ad56-962bf9bec1a2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "1528bf29-e941-4a4c-ad56-962bf9bec1a2") },
		{ "766b438c-646b-41c1-8dee-9fc0ff7b37d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "cf07a222-b6e6-46ab-a490-f77f774cdc6c"}, { new NonTerminator(NonTerminatorType.IdentifierList), "56e8abe4-943b-47d2-80f7-181a6bae7fad"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "766b438c-646b-41c1-8dee-9fc0ff7b37d5") },
		{ "56e8abe4-943b-47d2-80f7-181a6bae7fad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "bae1295c-50d3-4a7f-bc69-1fd42c241298"}, { new Terminator(DelimiterType.Comma), "11c0852f-9d83-4729-b039-69e05b9e04df"},}, new Dictionary<Terminator, ReduceInformation>{ }, "56e8abe4-943b-47d2-80f7-181a6bae7fad") },
		{ "de0d1ef0-edc6-45a7-81a7-9e48f7d4cb20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "e00a738e-b797-4cac-a4f5-16a95e247e49"}, { new Terminator(KeywordType.Integer), "a96775ea-398e-416c-803c-842482059a00"}, { new Terminator(KeywordType.Real), "815f5e68-35bd-4247-966e-8b3824c407dd"}, { new Terminator(KeywordType.Boolean), "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0"}, { new Terminator(KeywordType.Character), "90564982-90b1-4eaf-a3c5-467d173f730a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "de0d1ef0-edc6-45a7-81a7-9e48f7d4cb20") },
		{ "5f14fe2b-b48c-485f-9cc2-b60c8e5db053", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f55db05c-aa12-4436-a6d2-a48d864a2d58"}, { new Terminator(DelimiterType.Semicolon), "8e49eb1f-4a76-463e-b84f-dcba0c4789fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5f14fe2b-b48c-485f-9cc2-b60c8e5db053") },
		{ "4812c3bb-21e1-4cc1-93e8-bfbcc92e6142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "4812c3bb-21e1-4cc1-93e8-bfbcc92e6142") },
		{ "539de0b8-8967-4976-b099-82693968ec05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "6b9faa9c-2aaf-4fff-a5d9-1b41ae4a0469"}, { new Terminator(DelimiterType.Comma), "66896f28-a1c2-414d-ab5a-1f0dc9687ddb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "539de0b8-8967-4976-b099-82693968ec05") },
		{ "aab716cb-df44-4886-9051-967c381a29e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "043dc7ad-5735-4b93-9c46-1ed9eaef2350"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aab716cb-df44-4886-9051-967c381a29e9") },
		{ "11a44452-1d7d-484f-a590-318fc272a984", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "62146b9a-3c9b-42a9-9a21-026b166e74ea"}, { new NonTerminator(NonTerminatorType.Term), "a4b933b3-7375-43b7-b236-50d1cc5269ee"}, { new NonTerminator(NonTerminatorType.Factor), "e000d176-1682-4847-8e70-ec982c4ae4fd"}, { Terminator.NumberTerminator, "725b2576-df41-4ca4-b8e5-62f9e4fdf931"}, { new NonTerminator(NonTerminatorType.Variable), "4c414e6c-c247-4826-b6d6-06609811edab"}, { new Terminator(DelimiterType.LeftParenthesis), "12a43d38-ab32-4511-a884-0fcbe38b57c6"}, { Terminator.IdentifierTerminator, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe"}, { new Terminator(KeywordType.Not), "8e0cd647-2a0c-4061-b5f1-e09beb035070"}, { new Terminator(OperatorType.Minus), "fc0dbccb-495a-4819-8333-766ee05b8fa0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "11a44452-1d7d-484f-a590-318fc272a984") },
		{ "83089975-522a-4cc8-8043-15a6e7df9db6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2b64855d-18bb-4edd-94e2-8f1a8918d3f8"}, { new NonTerminator(NonTerminatorType.Factor), "79707115-1161-4189-b15c-42be5f0e7184"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "83089975-522a-4cc8-8043-15a6e7df9db6") },
		{ "30390509-2485-4027-b468-7f5a06f45911", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7c02d04d-b4e0-4c31-894f-03f0011fa886"}, { Terminator.NumberTerminator, "b0da74db-1321-4ad9-9e15-008c14315e6d"}, { new NonTerminator(NonTerminatorType.Variable), "084e5d87-68de-499f-9563-726dfa760000"}, { new Terminator(DelimiterType.LeftParenthesis), "7a97cd5c-a16c-4f18-a2eb-b23e04ccbb60"}, { Terminator.IdentifierTerminator, "bb7a33ac-07b6-4927-8505-fb4f90091683"}, { new Terminator(KeywordType.Not), "25d250a8-0d8f-4ec3-8eb9-cb5fdd592482"}, { new Terminator(OperatorType.Minus), "f1c47f00-204c-43e5-bfe6-6877e4b3ef48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "30390509-2485-4027-b468-7f5a06f45911") },
		{ "86010c6f-fdbd-4019-8d71-02ede82abc22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0c3dae47-e7d3-401c-835d-1d3bc7f92027"},}, new Dictionary<Terminator, ReduceInformation>{ }, "86010c6f-fdbd-4019-8d71-02ede82abc22") },
		{ "7606f153-120c-4355-ae8a-59dec9fc2258", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0a31c740-bbd5-40c9-a85e-eaba6cb4b383"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7606f153-120c-4355-ae8a-59dec9fc2258") },
		{ "78ecf1b9-59f7-46d4-aceb-a8fd852a9230", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "78ecf1b9-59f7-46d4-aceb-a8fd852a9230") },
		{ "80664dd8-50f0-4861-9f93-1f4772365f4b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "dac3ffdb-85a0-4d5f-97e3-5b5036dd7d63"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80664dd8-50f0-4861-9f93-1f4772365f4b") },
		{ "85199747-3d53-403c-9e46-62ca7127d2ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "85199747-3d53-403c-9e46-62ca7127d2ef") },
		{ "84cdc13a-c47c-4a6c-8da5-bcab90220fee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "84cdc13a-c47c-4a6c-8da5-bcab90220fee") },
		{ "67a0b1df-6860-454e-a4d5-a0f5a53107c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "67a0b1df-6860-454e-a4d5-a0f5a53107c6") },
		{ "7c137af1-0a27-40a0-b473-5b4f68a14794", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ebcebcb6-4d9a-49ec-bf6c-1bd5b37ee128"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7c137af1-0a27-40a0-b473-5b4f68a14794") },
		{ "14e906d4-b447-487c-a383-726a031a8886", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "572be756-b4c0-4f63-a551-bd1ec8ec4e71"}, { new NonTerminator(NonTerminatorType.Term), "63704df3-105d-49f9-9174-13614e2c99d3"}, { new NonTerminator(NonTerminatorType.Factor), "970e0165-64a5-4ec7-bae2-85181b442f90"}, { Terminator.NumberTerminator, "74f1d1b0-fcdb-43c9-935b-23f9465976e9"}, { new NonTerminator(NonTerminatorType.Variable), "835216be-d46c-4df4-84b4-d5785befeefa"}, { new Terminator(DelimiterType.LeftParenthesis), "9cd5687e-6fd5-4af7-8d99-b24d3988bf19"}, { Terminator.IdentifierTerminator, "d865fbc1-867d-4767-966b-c7d31b869fcb"}, { new Terminator(KeywordType.Not), "2f4d5cd4-5645-4832-801d-6e0d02c1cec6"}, { new Terminator(OperatorType.Minus), "a5118873-6eca-4d2c-b428-528624cff241"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14e906d4-b447-487c-a383-726a031a8886") },
		{ "b10ec06e-38db-459f-bdf5-d8073ce98bf3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "462e2f93-98de-4378-ae3d-ffc2c0d9a95d"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b10ec06e-38db-459f-bdf5-d8073ce98bf3") },
		{ "097a5289-eae6-4e48-989f-82cd5230ec8f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "974d0595-f748-48c0-a5b4-b486d846e45f"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "097a5289-eae6-4e48-989f-82cd5230ec8f") },
		{ "f50bc9a0-9e95-4ac1-905e-663ec0496cf2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e96b1a1a-0808-4ecd-af91-8ec224a86e8d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f50bc9a0-9e95-4ac1-905e-663ec0496cf2") },
		{ "93476f7b-7fdf-4e7a-8d96-dfe48640e626", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "b8bb0157-bc63-4942-91f4-9233fafe3781"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "93476f7b-7fdf-4e7a-8d96-dfe48640e626") },
		{ "130b533f-8e3f-4449-99b7-a5fe44f6df4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "130b533f-8e3f-4449-99b7-a5fe44f6df4c") },
		{ "e8d686bd-33cb-48f4-86a5-6fa0e51874ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a3c93a3b-646f-4887-822c-3500d38b293f"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e8d686bd-33cb-48f4-86a5-6fa0e51874ef") },
		{ "055269e0-0802-46b2-aad9-99fa3cae44ac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "055269e0-0802-46b2-aad9-99fa3cae44ac") },
		{ "7ff693b4-56b3-4bbf-be07-a487463a04bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7ff693b4-56b3-4bbf-be07-a487463a04bd") },
		{ "96f4075f-ecea-41f2-9962-ca38daaf80e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "96f4075f-ecea-41f2-9962-ca38daaf80e6") },
		{ "49111152-c351-4b8f-9641-f66a8cc03a71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "c8169d3f-4f6e-4ffd-b2fa-d9572c21287c"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "49111152-c351-4b8f-9641-f66a8cc03a71") },
		{ "2c643f3e-c602-4575-b2c4-e44565fc735e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "d094a416-5c17-459c-a4b6-6467bb073bd9"}, { new NonTerminator(NonTerminatorType.Term), "013a5a51-0078-4a9b-a692-b3ef7472942f"}, { new NonTerminator(NonTerminatorType.Factor), "e88da263-1b6c-40f3-b079-3d0fbeecb688"}, { Terminator.NumberTerminator, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7"}, { new NonTerminator(NonTerminatorType.Variable), "1abee674-834b-4b97-be87-2a3e0ae51d41"}, { new Terminator(DelimiterType.LeftParenthesis), "c65bb879-4e29-4e3c-a0a9-41f7a1c95045"}, { Terminator.IdentifierTerminator, "5970ac84-e42a-4e2f-af05-f2bf69693b8a"}, { new Terminator(KeywordType.Not), "b6a52b6c-2e7a-41cf-86e6-5261440d939e"}, { new Terminator(OperatorType.Minus), "27db342f-7a23-4075-b704-036f6de0ae54"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2c643f3e-c602-4575-b2c4-e44565fc735e") },
		{ "be234840-b527-4931-b33c-d59e92d9c04c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "4dbe2466-9f3a-4cf6-b388-9e1dd6af1f6a"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be234840-b527-4931-b33c-d59e92d9c04c") },
		{ "c0662fa7-53cc-443b-869c-32aaf59fa3fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "920df187-9228-4a06-b416-4ce16a73fcec"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c0662fa7-53cc-443b-869c-32aaf59fa3fe") },
		{ "1f5ad0f3-9700-4202-9347-6d5b90058933", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "00ddf2e1-4329-479d-a343-af44e07500a7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1f5ad0f3-9700-4202-9347-6d5b90058933") },
		{ "452e8264-72d7-4819-bb61-d49c33976c1f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "dc38b14a-094a-4e3e-9d16-7238a4c48bf3"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "452e8264-72d7-4819-bb61-d49c33976c1f") },
		{ "d1eb42cd-85e2-4165-a008-b68c8a312337", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d1eb42cd-85e2-4165-a008-b68c8a312337") },
		{ "d068a8bc-d79e-41fd-946b-e45864d58a1b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "3e3c2116-ac66-4a4a-92bf-55c7563c8917"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d068a8bc-d79e-41fd-946b-e45864d58a1b") },
		{ "018b5b46-29d9-475c-b426-c6317b50dd8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "018b5b46-29d9-475c-b426-c6317b50dd8c") },
		{ "0190cdbf-ca19-4e97-b786-5c121f2da4a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0190cdbf-ca19-4e97-b786-5c121f2da4a8") },
		{ "7028e224-1791-4dcb-b0c5-5aab5913c4ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "058a45b2-18a0-414a-9424-c665c117bf9b"}, { new Terminator(KeywordType.Else), "fa97fe6d-0a8a-4d26-b499-c373eca21446"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "7028e224-1791-4dcb-b0c5-5aab5913c4ee") },
		{ "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "945d06e7-7b12-4c54-81dd-023e193942e5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36") },
		{ "d74ee885-45f2-4595-a66a-53e932d8391f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "a4b44921-fada-49f3-97f5-60ab56c15883"}, { new NonTerminator(NonTerminatorType.IdVarPart), "2b607612-2ceb-45fd-b56a-e86a62d2f563"}, { new Terminator(DelimiterType.LeftSquareBracket), "4160990e-129b-441e-9e5b-4a514c45faef"}, { new Terminator(DelimiterType.LeftParenthesis), "ac2f6c55-9cf5-44b4-a853-d2c83054f159"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "d74ee885-45f2-4595-a66a-53e932d8391f") },
		{ "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0") },
		{ "b8794192-f4c9-4a1b-916f-bfaf49f43208", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "b8794192-f4c9-4a1b-916f-bfaf49f43208") },
		{ "f6810f84-1a46-4894-8c44-c2f8068e5c2f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "58bd421f-ac72-429d-b7f4-589ee8a03f0b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9015fb18-d537-480b-851b-9ddc84c18490"}, { new NonTerminator(NonTerminatorType.Term), "c428bc45-4c85-4c95-8c80-925e92f7c52e"}, { new NonTerminator(NonTerminatorType.Factor), "a9eca599-2434-41b1-93f5-2f6966bc3580"}, { Terminator.NumberTerminator, "ae79fdb0-d398-4ac7-b604-069a7d67076a"}, { new NonTerminator(NonTerminatorType.Variable), "50dd712b-3a18-4419-9af4-91e2b902a793"}, { new Terminator(DelimiterType.LeftParenthesis), "d86ea2c0-1043-4b2f-80e4-8aeb7f37b8aa"}, { Terminator.IdentifierTerminator, "546e5c99-a3fb-4bcb-98ac-01aacabb3d51"}, { new Terminator(KeywordType.Not), "eab617f6-a74a-4e0e-86a6-6ab31735be87"}, { new Terminator(OperatorType.Minus), "8adceea0-905c-42b1-879f-562bbb8bcd94"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f6810f84-1a46-4894-8c44-c2f8068e5c2f") },
		{ "954e1e88-c806-45af-84bf-a00026c87a9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "1651d727-aec3-4ae4-b78b-c0708d1e40a1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "954e1e88-c806-45af-84bf-a00026c87a9c") },
		{ "52ecab85-fa9a-45b6-942d-b6de60d7087c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "cff2a62c-e58a-439f-a2d3-b505045dd118"}, { new NonTerminator(NonTerminatorType.Statement), "669b7b94-f3e0-493a-8de2-511a254a494b"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "52ecab85-fa9a-45b6-942d-b6de60d7087c") },
		{ "98c75aa1-83a7-4ef5-ab87-78fddde6d44e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "6f85dd3c-34b7-4222-b2ee-394ad69e1278"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "98c75aa1-83a7-4ef5-ab87-78fddde6d44e") },
		{ "a2432249-8085-4c99-9c29-761396d84e3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ce325b26-efe9-45e0-ab22-7c9e1441e089"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a2432249-8085-4c99-9c29-761396d84e3d") },
		{ "428e3613-c455-41cc-af27-0a1c2559081c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "428e3613-c455-41cc-af27-0a1c2559081c") },
		{ "1a7adbd9-96de-48b1-825b-50f70daf617b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1a7adbd9-96de-48b1-825b-50f70daf617b") },
		{ "b53dfb08-0712-473a-9b0d-55eea1a5a3d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b53dfb08-0712-473a-9b0d-55eea1a5a3d1") },
		{ "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9194cb52-230d-422d-bc52-c84efb117a28"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b") },
		{ "90d9d809-9879-4ad3-82e3-6e2d4dd026e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "73687768-7d80-447d-8936-3396891a1d23"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c4ba7238-2d19-428b-84fa-5a4f261a5ca1"}, { new Terminator(DelimiterType.LeftSquareBracket), "a4855f4a-33e0-485d-8ad7-1cd2ed98939a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4") },
		{ "e074f32e-891a-483b-bafe-b284c053fe26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1e97c2b7-66b2-4002-8dd6-1ace0afb1874"}, { Terminator.NumberTerminator, "1a7adbd9-96de-48b1-825b-50f70daf617b"}, { new NonTerminator(NonTerminatorType.Variable), "b53dfb08-0712-473a-9b0d-55eea1a5a3d1"}, { new Terminator(DelimiterType.LeftParenthesis), "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b"}, { Terminator.IdentifierTerminator, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4"}, { new Terminator(KeywordType.Not), "e074f32e-891a-483b-bafe-b284c053fe26"}, { new Terminator(OperatorType.Minus), "7f436b2a-1c49-4827-abce-bf45864feb99"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e074f32e-891a-483b-bafe-b284c053fe26") },
		{ "7f436b2a-1c49-4827-abce-bf45864feb99", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fb302b83-cb37-46c1-af4a-37f788824386"}, { Terminator.NumberTerminator, "1a7adbd9-96de-48b1-825b-50f70daf617b"}, { new NonTerminator(NonTerminatorType.Variable), "b53dfb08-0712-473a-9b0d-55eea1a5a3d1"}, { new Terminator(DelimiterType.LeftParenthesis), "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b"}, { Terminator.IdentifierTerminator, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4"}, { new Terminator(KeywordType.Not), "e074f32e-891a-483b-bafe-b284c053fe26"}, { new Terminator(OperatorType.Minus), "7f436b2a-1c49-4827-abce-bf45864feb99"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7f436b2a-1c49-4827-abce-bf45864feb99") },
		{ "b03c1c3d-d00e-4d4a-922c-4eca64816f52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "122ff222-3ee7-48f0-afe3-e9ae0f6b0102"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b03c1c3d-d00e-4d4a-922c-4eca64816f52") },
		{ "4640daab-e55c-4851-882c-e1e2b04689d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "4640daab-e55c-4851-882c-e1e2b04689d3") },
		{ "395d0cb9-d970-434b-aeee-fb8d7e471598", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "395d0cb9-d970-434b-aeee-fb8d7e471598") },
		{ "dedcb900-f845-408f-a097-1a62c07b2b38", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "35da2711-3064-4216-92ad-7e1e4e2e098f"}, { new NonTerminator(NonTerminatorType.Term), "84da7458-06e5-454a-a777-11d346958250"}, { new NonTerminator(NonTerminatorType.Factor), "d3912847-97a1-465d-9cd7-6dbba61cfbd7"}, { Terminator.NumberTerminator, "3c073275-f254-4aca-987c-1934f4682c0c"}, { new NonTerminator(NonTerminatorType.Variable), "b1a86855-464b-4b79-8cc2-bbfb5e375f74"}, { new Terminator(DelimiterType.LeftParenthesis), "966e3e24-a183-4c9b-83ef-09b1f41dd824"}, { Terminator.IdentifierTerminator, "3616b1e4-3979-4495-8908-afa295f8f030"}, { new Terminator(KeywordType.Not), "2572d6b4-5b66-47c2-bcaa-f40003515d7a"}, { new Terminator(OperatorType.Minus), "70dc3b23-0ce9-4ded-a0b3-1e683d782b30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dedcb900-f845-408f-a097-1a62c07b2b38") },
		{ "aaa50ef2-c576-441a-8a35-dfa53e223083", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "a919d61f-8d45-4e52-a7f4-9a885c6963ee"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aaa50ef2-c576-441a-8a35-dfa53e223083") },
		{ "4cc50272-72e3-4af7-b89d-8b06e7dddf37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "63440130-419e-4e5d-9d83-8113d8192a53"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4cc50272-72e3-4af7-b89d-8b06e7dddf37") },
		{ "a7237bd4-1e47-42a4-ace0-1e07ed47c5a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d7215f09-f808-4b1a-83cc-a573ac62efbd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a7237bd4-1e47-42a4-ace0-1e07ed47c5a3") },
		{ "4baa2207-1c00-4b57-884d-227000c3348e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "df93fb21-c600-4153-b7c0-888ac328c76f"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4baa2207-1c00-4b57-884d-227000c3348e") },
		{ "488a8f93-38d3-4aae-a518-9898727871a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "488a8f93-38d3-4aae-a518-9898727871a1") },
		{ "e68ab851-7f93-41e2-a07b-e069cf268c0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0f072488-a808-4c35-b6ba-63dabf98748c"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e68ab851-7f93-41e2-a07b-e069cf268c0a") },
		{ "cbbbcb62-bac0-431a-95ba-e175acf5d159", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "cbbbcb62-bac0-431a-95ba-e175acf5d159") },
		{ "b9fd39ed-e892-4c6d-a3a4-d82899ce6f3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b9fd39ed-e892-4c6d-a3a4-d82899ce6f3b") },
		{ "8a1c2190-7063-4afa-b7a4-d8e875d65598", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6d02688a-de60-4297-b54b-481a89243375"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8a1c2190-7063-4afa-b7a4-d8e875d65598") },
		{ "5b9df392-de08-4a85-80a6-9112c10a671f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1b167b1f-e166-404d-819d-43695835d37f"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b9df392-de08-4a85-80a6-9112c10a671f") },
		{ "e680f46b-abe5-483e-823e-ba060964f9fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "2dccf16b-43ed-4645-895b-8a0b8b8095d3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e680f46b-abe5-483e-823e-ba060964f9fc") },
		{ "9c417c34-245c-4d71-ac9b-a095bc19f366", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "ccc3b815-2dae-40df-8072-40a8db16105f"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "d4b26683-aafe-46ef-9354-03f9f918e2bd"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "9c417c34-245c-4d71-ac9b-a095bc19f366") },
		{ "9b515565-da09-46ce-aa9d-2a7969ad537c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "244157d5-9074-40a6-8598-3a1c9f695f91"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9b515565-da09-46ce-aa9d-2a7969ad537c") },
		{ "ae5fa63b-fa83-4af1-a392-77c6c28fc8c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "ae5fa63b-fa83-4af1-a392-77c6c28fc8c9") },
		{ "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557") },
		{ "44175282-e122-4250-b19e-9ab68843ffcf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "44175282-e122-4250-b19e-9ab68843ffcf") },
		{ "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "426f9d4c-7614-44e3-8e62-d208380de9d2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f") },
		{ "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "ce2239a7-8851-42be-8fa7-a18885c27253"}, { new NonTerminator(NonTerminatorType.IdVarPart), "8224c264-0dfd-4b8c-a6a4-a6fc1a1d3745"}, { new Terminator(DelimiterType.LeftSquareBracket), "40df73c3-5748-4ab7-89ab-6da7ea68b048"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d") },
		{ "3668d476-b4f2-445a-8830-e21aafcd18fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6fb2661b-b2c5-4776-b978-7827ff761381"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3668d476-b4f2-445a-8830-e21aafcd18fc") },
		{ "5fbeab4a-5be6-458b-8366-e945e9d1561d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e3d428e8-3978-42ba-a7ac-53ec7ab693a0"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5fbeab4a-5be6-458b-8366-e945e9d1561d") },
		{ "a80d1e17-6b63-488b-9d67-f75ec27b859d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "a80d1e17-6b63-488b-9d67-f75ec27b859d") },
		{ "54b3ebdc-53af-4829-9628-3af934c9bfa6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "98cb7770-1855-411b-9403-d67eacf23a7d"}, { new NonTerminator(NonTerminatorType.Statement), "669b7b94-f3e0-493a-8de2-511a254a494b"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "54b3ebdc-53af-4829-9628-3af934c9bfa6") },
		{ "4b931270-e39b-4d43-8cff-da0fa97ae435", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "4b931270-e39b-4d43-8cff-da0fa97ae435") },
		{ "8e49eb1f-4a76-463e-b84f-dcba0c4789fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "e6f21cdb-496d-4089-96ef-8a0679b17e96"}, { new NonTerminator(NonTerminatorType.VarParameter), "60ba72c9-f75f-4c97-b688-5541f9960d6f"}, { new NonTerminator(NonTerminatorType.ValueParameter), "1528bf29-e941-4a4c-ad56-962bf9bec1a2"}, { new Terminator(KeywordType.Var), "766b438c-646b-41c1-8dee-9fc0ff7b37d5"}, { new NonTerminator(NonTerminatorType.IdentifierList), "56e8abe4-943b-47d2-80f7-181a6bae7fad"}, { Terminator.IdentifierTerminator, "280edb8d-540f-42a6-be37-39effb326c4e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8e49eb1f-4a76-463e-b84f-dcba0c4789fa") },
		{ "cf07a222-b6e6-46ab-a490-f77f774cdc6c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "cf07a222-b6e6-46ab-a490-f77f774cdc6c") },
		{ "bae1295c-50d3-4a7f-bc69-1fd42c241298", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "498c8ae3-63c2-4475-8ccb-bbf380a0ed20"}, { new Terminator(KeywordType.Integer), "e2b4b868-4874-4254-b2c2-9e9983695c23"}, { new Terminator(KeywordType.Real), "2f0945cd-fefe-4964-839c-75413036309a"}, { new Terminator(KeywordType.Boolean), "de54a0ab-1392-45c9-83b5-5c058254bfc8"}, { new Terminator(KeywordType.Character), "f8172a17-ee31-45a4-b13a-7c8578c3328c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bae1295c-50d3-4a7f-bc69-1fd42c241298") },
		{ "e00a738e-b797-4cac-a4f5-16a95e247e49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "e00a738e-b797-4cac-a4f5-16a95e247e49") },
		{ "f55db05c-aa12-4436-a6d2-a48d864a2d58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "f55db05c-aa12-4436-a6d2-a48d864a2d58") },
		{ "6b9faa9c-2aaf-4fff-a5d9-1b41ae4a0469", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "e6724654-492f-4731-ae8e-7d1cb7256ddd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6b9faa9c-2aaf-4fff-a5d9-1b41ae4a0469") },
		{ "66896f28-a1c2-414d-ab5a-1f0dc9687ddb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "8b1afb8d-01ba-4806-9586-24cdf08f2009"},}, new Dictionary<Terminator, ReduceInformation>{ }, "66896f28-a1c2-414d-ab5a-1f0dc9687ddb") },
		{ "043dc7ad-5735-4b93-9c46-1ed9eaef2350", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "e7d27dfa-4906-42dc-9f2c-13bce59cbe73"},}, new Dictionary<Terminator, ReduceInformation>{ }, "043dc7ad-5735-4b93-9c46-1ed9eaef2350") },
		{ "62146b9a-3c9b-42a9-9a21-026b166e74ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "b80a3b3f-ef3b-4da3-a18c-8dd45f2c8226"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "62146b9a-3c9b-42a9-9a21-026b166e74ea") },
		{ "a4b933b3-7375-43b7-b236-50d1cc5269ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "656702ea-dabc-404c-8c60-afcd411f04e3"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a4b933b3-7375-43b7-b236-50d1cc5269ee") },
		{ "e000d176-1682-4847-8e70-ec982c4ae4fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e000d176-1682-4847-8e70-ec982c4ae4fd") },
		{ "725b2576-df41-4ca4-b8e5-62f9e4fdf931", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "725b2576-df41-4ca4-b8e5-62f9e4fdf931") },
		{ "4c414e6c-c247-4826-b6d6-06609811edab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4c414e6c-c247-4826-b6d6-06609811edab") },
		{ "12a43d38-ab32-4511-a884-0fcbe38b57c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ce59caf4-1fe4-4ba0-ad72-a7310bb218db"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "12a43d38-ab32-4511-a884-0fcbe38b57c6") },
		{ "d1647a8b-675c-4c9d-81f1-e6a744eb5efe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "897c1db1-c2c5-4ac8-8751-d4f5cc1cf135"}, { new NonTerminator(NonTerminatorType.IdVarPart), "a260110c-5289-4a9e-bedf-c13cfc5efabe"}, { new Terminator(DelimiterType.LeftSquareBracket), "ab4b00e1-3b8c-487a-a0cf-426853bba2f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe") },
		{ "8e0cd647-2a0c-4061-b5f1-e09beb035070", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "08928f2c-0dc0-4956-9597-9dbfdf8d04f7"}, { Terminator.NumberTerminator, "725b2576-df41-4ca4-b8e5-62f9e4fdf931"}, { new NonTerminator(NonTerminatorType.Variable), "4c414e6c-c247-4826-b6d6-06609811edab"}, { new Terminator(DelimiterType.LeftParenthesis), "12a43d38-ab32-4511-a884-0fcbe38b57c6"}, { Terminator.IdentifierTerminator, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe"}, { new Terminator(KeywordType.Not), "8e0cd647-2a0c-4061-b5f1-e09beb035070"}, { new Terminator(OperatorType.Minus), "fc0dbccb-495a-4819-8333-766ee05b8fa0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8e0cd647-2a0c-4061-b5f1-e09beb035070") },
		{ "fc0dbccb-495a-4819-8333-766ee05b8fa0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "ec5068f6-63df-4ed0-a50f-38e25c74655c"}, { Terminator.NumberTerminator, "725b2576-df41-4ca4-b8e5-62f9e4fdf931"}, { new NonTerminator(NonTerminatorType.Variable), "4c414e6c-c247-4826-b6d6-06609811edab"}, { new Terminator(DelimiterType.LeftParenthesis), "12a43d38-ab32-4511-a884-0fcbe38b57c6"}, { Terminator.IdentifierTerminator, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe"}, { new Terminator(KeywordType.Not), "8e0cd647-2a0c-4061-b5f1-e09beb035070"}, { new Terminator(OperatorType.Minus), "fc0dbccb-495a-4819-8333-766ee05b8fa0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fc0dbccb-495a-4819-8333-766ee05b8fa0") },
		{ "2b64855d-18bb-4edd-94e2-8f1a8918d3f8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "30390509-2485-4027-b468-7f5a06f45911"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2b64855d-18bb-4edd-94e2-8f1a8918d3f8") },
		{ "7c02d04d-b4e0-4c31-894f-03f0011fa886", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "7c02d04d-b4e0-4c31-894f-03f0011fa886") },
		{ "0c3dae47-e7d3-401c-835d-1d3bc7f92027", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0c3dae47-e7d3-401c-835d-1d3bc7f92027") },
		{ "0a31c740-bbd5-40c9-a85e-eaba6cb4b383", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "83a11cfb-a850-4f3f-b9e1-7d93db67c90a"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0a31c740-bbd5-40c9-a85e-eaba6cb4b383") },
		{ "dac3ffdb-85a0-4d5f-97e3-5b5036dd7d63", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "a5c43f99-cb84-4ab9-9565-65dc94316f3d"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dac3ffdb-85a0-4d5f-97e3-5b5036dd7d63") },
		{ "ebcebcb6-4d9a-49ec-bf6c-1bd5b37ee128", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "ebcebcb6-4d9a-49ec-bf6c-1bd5b37ee128") },
		{ "572be756-b4c0-4f63-a551-bd1ec8ec4e71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "028551ef-6ec6-4120-bb8a-dbad945234e0"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "572be756-b4c0-4f63-a551-bd1ec8ec4e71") },
		{ "63704df3-105d-49f9-9174-13614e2c99d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "e3f07366-8e2b-4c79-bd53-16fc10a10be1"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "63704df3-105d-49f9-9174-13614e2c99d3") },
		{ "970e0165-64a5-4ec7-bae2-85181b442f90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "970e0165-64a5-4ec7-bae2-85181b442f90") },
		{ "74f1d1b0-fcdb-43c9-935b-23f9465976e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "74f1d1b0-fcdb-43c9-935b-23f9465976e9") },
		{ "835216be-d46c-4df4-84b4-d5785befeefa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "835216be-d46c-4df4-84b4-d5785befeefa") },
		{ "9cd5687e-6fd5-4af7-8d99-b24d3988bf19", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "1d2c865b-9298-42f1-9cd5-0bab8dae1544"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9cd5687e-6fd5-4af7-8d99-b24d3988bf19") },
		{ "d865fbc1-867d-4767-966b-c7d31b869fcb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "1dc0db37-59f1-4f22-86f5-c6f575ad3a5b"}, { new NonTerminator(NonTerminatorType.IdVarPart), "46ef772a-aba5-4efe-9628-f0a760243a89"}, { new Terminator(DelimiterType.LeftSquareBracket), "551be8c9-aad7-4461-8597-0dc4a3208189"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d865fbc1-867d-4767-966b-c7d31b869fcb") },
		{ "2f4d5cd4-5645-4832-801d-6e0d02c1cec6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3bf8b7a5-ec92-449a-af8f-872863834f09"}, { Terminator.NumberTerminator, "74f1d1b0-fcdb-43c9-935b-23f9465976e9"}, { new NonTerminator(NonTerminatorType.Variable), "835216be-d46c-4df4-84b4-d5785befeefa"}, { new Terminator(DelimiterType.LeftParenthesis), "9cd5687e-6fd5-4af7-8d99-b24d3988bf19"}, { Terminator.IdentifierTerminator, "d865fbc1-867d-4767-966b-c7d31b869fcb"}, { new Terminator(KeywordType.Not), "2f4d5cd4-5645-4832-801d-6e0d02c1cec6"}, { new Terminator(OperatorType.Minus), "a5118873-6eca-4d2c-b428-528624cff241"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2f4d5cd4-5645-4832-801d-6e0d02c1cec6") },
		{ "a5118873-6eca-4d2c-b428-528624cff241", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6fcc126d-82e9-4602-8775-01cce0ecc602"}, { Terminator.NumberTerminator, "74f1d1b0-fcdb-43c9-935b-23f9465976e9"}, { new NonTerminator(NonTerminatorType.Variable), "835216be-d46c-4df4-84b4-d5785befeefa"}, { new Terminator(DelimiterType.LeftParenthesis), "9cd5687e-6fd5-4af7-8d99-b24d3988bf19"}, { Terminator.IdentifierTerminator, "d865fbc1-867d-4767-966b-c7d31b869fcb"}, { new Terminator(KeywordType.Not), "2f4d5cd4-5645-4832-801d-6e0d02c1cec6"}, { new Terminator(OperatorType.Minus), "a5118873-6eca-4d2c-b428-528624cff241"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5118873-6eca-4d2c-b428-528624cff241") },
		{ "462e2f93-98de-4378-ae3d-ffc2c0d9a95d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "097a5289-eae6-4e48-989f-82cd5230ec8f"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "462e2f93-98de-4378-ae3d-ffc2c0d9a95d") },
		{ "974d0595-f748-48c0-a5b4-b486d846e45f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "974d0595-f748-48c0-a5b4-b486d846e45f") },
		{ "e96b1a1a-0808-4ecd-af91-8ec224a86e8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "e96b1a1a-0808-4ecd-af91-8ec224a86e8d") },
		{ "b8bb0157-bc63-4942-91f4-9233fafe3781", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4b153968-6d82-4b65-9f8a-204c929b3cfe"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b8bb0157-bc63-4942-91f4-9233fafe3781") },
		{ "a3c93a3b-646f-4887-822c-3500d38b293f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "04ce043b-fa3d-497d-9fb5-75617ab702c7"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3c93a3b-646f-4887-822c-3500d38b293f") },
		{ "c8169d3f-4f6e-4ffd-b2fa-d9572c21287c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "c8169d3f-4f6e-4ffd-b2fa-d9572c21287c") },
		{ "d094a416-5c17-459c-a4b6-6467bb073bd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "c6c879a7-7627-49af-ab21-54ae0dfd8fc3"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "d094a416-5c17-459c-a4b6-6467bb073bd9") },
		{ "013a5a51-0078-4a9b-a692-b3ef7472942f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4c6570dd-7342-4595-9139-6fcc9bafb635"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "013a5a51-0078-4a9b-a692-b3ef7472942f") },
		{ "e88da263-1b6c-40f3-b079-3d0fbeecb688", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e88da263-1b6c-40f3-b079-3d0fbeecb688") },
		{ "bb504128-cb1f-4fcd-b062-9a557fe4b3c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7") },
		{ "1abee674-834b-4b97-be87-2a3e0ae51d41", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1abee674-834b-4b97-be87-2a3e0ae51d41") },
		{ "c65bb879-4e29-4e3c-a0a9-41f7a1c95045", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d11dfd34-a0a4-45c6-b41c-0fe59587ca00"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c65bb879-4e29-4e3c-a0a9-41f7a1c95045") },
		{ "5970ac84-e42a-4e2f-af05-f2bf69693b8a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "83b5acd7-84e8-4145-b4ad-ec2ff376ae5f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "aacab2d0-5282-4343-98fc-f64508a9e77a"}, { new Terminator(DelimiterType.LeftSquareBracket), "bc942d8a-fd96-41d8-8e50-5d1fba8d404c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5970ac84-e42a-4e2f-af05-f2bf69693b8a") },
		{ "b6a52b6c-2e7a-41cf-86e6-5261440d939e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7de0111a-dfeb-4b5b-aba0-8111d825f750"}, { Terminator.NumberTerminator, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7"}, { new NonTerminator(NonTerminatorType.Variable), "1abee674-834b-4b97-be87-2a3e0ae51d41"}, { new Terminator(DelimiterType.LeftParenthesis), "c65bb879-4e29-4e3c-a0a9-41f7a1c95045"}, { Terminator.IdentifierTerminator, "5970ac84-e42a-4e2f-af05-f2bf69693b8a"}, { new Terminator(KeywordType.Not), "b6a52b6c-2e7a-41cf-86e6-5261440d939e"}, { new Terminator(OperatorType.Minus), "27db342f-7a23-4075-b704-036f6de0ae54"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b6a52b6c-2e7a-41cf-86e6-5261440d939e") },
		{ "27db342f-7a23-4075-b704-036f6de0ae54", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "30f06511-3135-4567-971b-1340cd6617a0"}, { Terminator.NumberTerminator, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7"}, { new NonTerminator(NonTerminatorType.Variable), "1abee674-834b-4b97-be87-2a3e0ae51d41"}, { new Terminator(DelimiterType.LeftParenthesis), "c65bb879-4e29-4e3c-a0a9-41f7a1c95045"}, { Terminator.IdentifierTerminator, "5970ac84-e42a-4e2f-af05-f2bf69693b8a"}, { new Terminator(KeywordType.Not), "b6a52b6c-2e7a-41cf-86e6-5261440d939e"}, { new Terminator(OperatorType.Minus), "27db342f-7a23-4075-b704-036f6de0ae54"},}, new Dictionary<Terminator, ReduceInformation>{ }, "27db342f-7a23-4075-b704-036f6de0ae54") },
		{ "4dbe2466-9f3a-4cf6-b388-9e1dd6af1f6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c0662fa7-53cc-443b-869c-32aaf59fa3fe"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "4dbe2466-9f3a-4cf6-b388-9e1dd6af1f6a") },
		{ "920df187-9228-4a06-b416-4ce16a73fcec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "920df187-9228-4a06-b416-4ce16a73fcec") },
		{ "00ddf2e1-4329-479d-a343-af44e07500a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "00ddf2e1-4329-479d-a343-af44e07500a7") },
		{ "dc38b14a-094a-4e3e-9d16-7238a4c48bf3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5e32f531-e627-4034-9560-2ade80ba4500"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc38b14a-094a-4e3e-9d16-7238a4c48bf3") },
		{ "3e3c2116-ac66-4a4a-92bf-55c7563c8917", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "7e4b4e03-71b0-46da-853b-03b24807b0f0"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3e3c2116-ac66-4a4a-92bf-55c7563c8917") },
		{ "058a45b2-18a0-414a-9424-c665c117bf9b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "058a45b2-18a0-414a-9424-c665c117bf9b") },
		{ "fa97fe6d-0a8a-4d26-b499-c373eca21446", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "9aebf022-347c-4065-8f28-47a67879c947"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "fa97fe6d-0a8a-4d26-b499-c373eca21446") },
		{ "945d06e7-7b12-4c54-81dd-023e193942e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "00cde644-c96f-4f62-ab38-363354730ff8"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "8b94a8a2-3347-4f1b-bcee-4b503291b55f"}, { new NonTerminator(NonTerminatorType.Term), "ce25b143-71c2-418e-8e4c-1e2f1a762a83"}, { new NonTerminator(NonTerminatorType.Factor), "2e69467b-75b2-43d4-ad3e-5362684cc4c1"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "945d06e7-7b12-4c54-81dd-023e193942e5") },
		{ "a4b44921-fada-49f3-97f5-60ab56c15883", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d451b2b0-da56-414e-b7e1-c71cba23f495"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "8b94a8a2-3347-4f1b-bcee-4b503291b55f"}, { new NonTerminator(NonTerminatorType.Term), "ce25b143-71c2-418e-8e4c-1e2f1a762a83"}, { new NonTerminator(NonTerminatorType.Factor), "2e69467b-75b2-43d4-ad3e-5362684cc4c1"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a4b44921-fada-49f3-97f5-60ab56c15883") },
		{ "ac2f6c55-9cf5-44b4-a853-d2c83054f159", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "5531e016-2693-447a-8395-b7a188ec650f"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac2f6c55-9cf5-44b4-a853-d2c83054f159") },
		{ "58bd421f-ac72-429d-b7f4-589ee8a03f0b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "502ad32f-85c0-4328-aa69-199378bf5013"},}, new Dictionary<Terminator, ReduceInformation>{ }, "58bd421f-ac72-429d-b7f4-589ee8a03f0b") },
		{ "1651d727-aec3-4ae4-b78b-c0708d1e40a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "8269c0b3-0051-4e2c-b2c6-2485ab13268d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1651d727-aec3-4ae4-b78b-c0708d1e40a1") },
		{ "cff2a62c-e58a-439f-a2d3-b505045dd118", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "73113c3f-17d9-4344-9a39-cecc8b70d2ac"}, { new Terminator(DelimiterType.Semicolon), "56452484-c5ea-4e76-9fdf-c2dbc7763e55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cff2a62c-e58a-439f-a2d3-b505045dd118") },
		{ "6f85dd3c-34b7-4222-b2ee-394ad69e1278", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b5687c72-830b-429d-a30c-76148d840e22"}, { new NonTerminator(NonTerminatorType.Factor), "428e3613-c455-41cc-af27-0a1c2559081c"}, { Terminator.NumberTerminator, "1a7adbd9-96de-48b1-825b-50f70daf617b"}, { new NonTerminator(NonTerminatorType.Variable), "b53dfb08-0712-473a-9b0d-55eea1a5a3d1"}, { new Terminator(DelimiterType.LeftParenthesis), "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b"}, { Terminator.IdentifierTerminator, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4"}, { new Terminator(KeywordType.Not), "e074f32e-891a-483b-bafe-b284c053fe26"}, { new Terminator(OperatorType.Minus), "7f436b2a-1c49-4827-abce-bf45864feb99"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f85dd3c-34b7-4222-b2ee-394ad69e1278") },
		{ "ce325b26-efe9-45e0-ab22-7c9e1441e089", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e44bd5be-d5bc-46b3-8078-e291cc91a7df"}, { Terminator.NumberTerminator, "1a7adbd9-96de-48b1-825b-50f70daf617b"}, { new NonTerminator(NonTerminatorType.Variable), "b53dfb08-0712-473a-9b0d-55eea1a5a3d1"}, { new Terminator(DelimiterType.LeftParenthesis), "d4ccc2ea-50a3-41bb-ab6d-5dd99a3df00b"}, { Terminator.IdentifierTerminator, "90d9d809-9879-4ad3-82e3-6e2d4dd026e4"}, { new Terminator(KeywordType.Not), "e074f32e-891a-483b-bafe-b284c053fe26"}, { new Terminator(OperatorType.Minus), "7f436b2a-1c49-4827-abce-bf45864feb99"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ce325b26-efe9-45e0-ab22-7c9e1441e089") },
		{ "9194cb52-230d-422d-bc52-c84efb117a28", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "78f99f90-ff30-40cd-83df-f7bc1950330a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9194cb52-230d-422d-bc52-c84efb117a28") },
		{ "73687768-7d80-447d-8936-3396891a1d23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "62917d15-21c0-4112-afdb-f18a1d4bcb5b"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73687768-7d80-447d-8936-3396891a1d23") },
		{ "c4ba7238-2d19-428b-84fa-5a4f261a5ca1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c4ba7238-2d19-428b-84fa-5a4f261a5ca1") },
		{ "a4855f4a-33e0-485d-8ad7-1cd2ed98939a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cf7dc292-45e2-4322-9730-55e10494526f"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a4855f4a-33e0-485d-8ad7-1cd2ed98939a") },
		{ "1e97c2b7-66b2-4002-8dd6-1ace0afb1874", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1e97c2b7-66b2-4002-8dd6-1ace0afb1874") },
		{ "fb302b83-cb37-46c1-af4a-37f788824386", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "fb302b83-cb37-46c1-af4a-37f788824386") },
		{ "35da2711-3064-4216-92ad-7e1e4e2e098f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "4b3f9459-fe04-461b-9ed0-2ec69678345a"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "35da2711-3064-4216-92ad-7e1e4e2e098f") },
		{ "84da7458-06e5-454a-a777-11d346958250", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3371f493-d8d4-441d-8e9d-64b1a1a3ffeb"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "84da7458-06e5-454a-a777-11d346958250") },
		{ "d3912847-97a1-465d-9cd7-6dbba61cfbd7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "d3912847-97a1-465d-9cd7-6dbba61cfbd7") },
		{ "3c073275-f254-4aca-987c-1934f4682c0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3c073275-f254-4aca-987c-1934f4682c0c") },
		{ "b1a86855-464b-4b79-8cc2-bbfb5e375f74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b1a86855-464b-4b79-8cc2-bbfb5e375f74") },
		{ "966e3e24-a183-4c9b-83ef-09b1f41dd824", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d0445134-77cc-4724-8954-cf34a23129e6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "966e3e24-a183-4c9b-83ef-09b1f41dd824") },
		{ "3616b1e4-3979-4495-8908-afa295f8f030", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "d530f6b4-3f5e-4b3f-8856-dabd17a50828"}, { new NonTerminator(NonTerminatorType.IdVarPart), "76e96605-607d-4447-9b54-bf5c6575760f"}, { new Terminator(DelimiterType.LeftSquareBracket), "6fcb77eb-8903-41e2-9a38-ea764fe684e3"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "3616b1e4-3979-4495-8908-afa295f8f030") },
		{ "2572d6b4-5b66-47c2-bcaa-f40003515d7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3cf42241-f0c2-4108-9110-baded553ef25"}, { Terminator.NumberTerminator, "3c073275-f254-4aca-987c-1934f4682c0c"}, { new NonTerminator(NonTerminatorType.Variable), "b1a86855-464b-4b79-8cc2-bbfb5e375f74"}, { new Terminator(DelimiterType.LeftParenthesis), "966e3e24-a183-4c9b-83ef-09b1f41dd824"}, { Terminator.IdentifierTerminator, "3616b1e4-3979-4495-8908-afa295f8f030"}, { new Terminator(KeywordType.Not), "2572d6b4-5b66-47c2-bcaa-f40003515d7a"}, { new Terminator(OperatorType.Minus), "70dc3b23-0ce9-4ded-a0b3-1e683d782b30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2572d6b4-5b66-47c2-bcaa-f40003515d7a") },
		{ "70dc3b23-0ce9-4ded-a0b3-1e683d782b30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "199684b4-cbcc-49de-95eb-2c63a9073a88"}, { Terminator.NumberTerminator, "3c073275-f254-4aca-987c-1934f4682c0c"}, { new NonTerminator(NonTerminatorType.Variable), "b1a86855-464b-4b79-8cc2-bbfb5e375f74"}, { new Terminator(DelimiterType.LeftParenthesis), "966e3e24-a183-4c9b-83ef-09b1f41dd824"}, { Terminator.IdentifierTerminator, "3616b1e4-3979-4495-8908-afa295f8f030"}, { new Terminator(KeywordType.Not), "2572d6b4-5b66-47c2-bcaa-f40003515d7a"}, { new Terminator(OperatorType.Minus), "70dc3b23-0ce9-4ded-a0b3-1e683d782b30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "70dc3b23-0ce9-4ded-a0b3-1e683d782b30") },
		{ "a919d61f-8d45-4e52-a7f4-9a885c6963ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4cc50272-72e3-4af7-b89d-8b06e7dddf37"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a919d61f-8d45-4e52-a7f4-9a885c6963ee") },
		{ "63440130-419e-4e5d-9d83-8113d8192a53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "63440130-419e-4e5d-9d83-8113d8192a53") },
		{ "d7215f09-f808-4b1a-83cc-a573ac62efbd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d7215f09-f808-4b1a-83cc-a573ac62efbd") },
		{ "df93fb21-c600-4153-b7c0-888ac328c76f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e3c61076-2a64-4b9b-918a-6e32acd31dc5"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df93fb21-c600-4153-b7c0-888ac328c76f") },
		{ "0f072488-a808-4c35-b6ba-63dabf98748c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "8ed3f13d-a162-470e-886a-684b814d96eb"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0f072488-a808-4c35-b6ba-63dabf98748c") },
		{ "6d02688a-de60-4297-b54b-481a89243375", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "6d02688a-de60-4297-b54b-481a89243375") },
		{ "1b167b1f-e166-404d-819d-43695835d37f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1b167b1f-e166-404d-819d-43695835d37f") },
		{ "2dccf16b-43ed-4645-895b-8a0b8b8095d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bd5d1ff4-dece-487d-b6ec-2784faed1742"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "cb5620f4-43e3-4e20-a089-373f30c4d741"}, { new NonTerminator(NonTerminatorType.Term), "460855ab-487d-4f1d-90e4-d922f0bd6520"}, { new NonTerminator(NonTerminatorType.Factor), "3a8d241f-315a-440c-a585-2cf6dee76653"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2dccf16b-43ed-4645-895b-8a0b8b8095d3") },
		{ "ccc3b815-2dae-40df-8072-40a8db16105f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "f58d81b4-32b7-4db6-b7e2-d7703194cd73"}, { new NonTerminator(NonTerminatorType.Term), "2826d44b-91bb-4f74-b2ee-d25438624989"}, { new NonTerminator(NonTerminatorType.Factor), "950fb149-1176-4711-8afb-9a7ebf5cbb8d"}, { Terminator.NumberTerminator, "579d451b-936b-48a4-b3fc-09a46555f5cd"}, { new NonTerminator(NonTerminatorType.Variable), "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf"}, { new Terminator(DelimiterType.LeftParenthesis), "ba3fc134-6a9a-4554-aa1c-6acba919674e"}, { Terminator.IdentifierTerminator, "fedef25c-c4e4-436a-9c87-f345407b58ab"}, { new Terminator(KeywordType.Not), "06dd1dba-097b-46c1-8406-acf688137867"}, { new Terminator(OperatorType.Minus), "ca6eb5a5-9781-4977-97bc-daba801a61fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ccc3b815-2dae-40df-8072-40a8db16105f") },
		{ "d4b26683-aafe-46ef-9354-03f9f918e2bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "d93dd101-59ac-4533-8f47-09910cfe1336"}, { new NonTerminator(NonTerminatorType.Factor), "ae5fa63b-fa83-4af1-a392-77c6c28fc8c9"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4b26683-aafe-46ef-9354-03f9f918e2bd") },
		{ "244157d5-9074-40a6-8598-3a1c9f695f91", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f956d88a-3fae-4b42-af7b-3cf613e05c32"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "244157d5-9074-40a6-8598-3a1c9f695f91") },
		{ "426f9d4c-7614-44e3-8e62-d208380de9d2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a061df0a-5c2b-4c7e-8ce0-e71f67da8415"},}, new Dictionary<Terminator, ReduceInformation>{ }, "426f9d4c-7614-44e3-8e62-d208380de9d2") },
		{ "ce2239a7-8851-42be-8fa7-a18885c27253", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "4d4f158f-9dac-4aed-ac99-af254890ac26"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ce2239a7-8851-42be-8fa7-a18885c27253") },
		{ "8224c264-0dfd-4b8c-a6a4-a6fc1a1d3745", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "8224c264-0dfd-4b8c-a6a4-a6fc1a1d3745") },
		{ "40df73c3-5748-4ab7-89ab-6da7ea68b048", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e79929ab-9cf5-43ae-8fa7-2031298c3d9b"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "40df73c3-5748-4ab7-89ab-6da7ea68b048") },
		{ "6fb2661b-b2c5-4776-b978-7827ff761381", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6fb2661b-b2c5-4776-b978-7827ff761381") },
		{ "e3d428e8-3978-42ba-a7ac-53ec7ab693a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e3d428e8-3978-42ba-a7ac-53ec7ab693a0") },
		{ "98cb7770-1855-411b-9403-d67eacf23a7d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "b7c2ead4-246b-4eee-9e9a-a5e1b8bbe083"}, { new Terminator(DelimiterType.Semicolon), "56452484-c5ea-4e76-9fdf-c2dbc7763e55"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98cb7770-1855-411b-9403-d67eacf23a7d") },
		{ "e6f21cdb-496d-4089-96ef-8a0679b17e96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "e6f21cdb-496d-4089-96ef-8a0679b17e96") },
		{ "498c8ae3-63c2-4475-8ccb-bbf380a0ed20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "498c8ae3-63c2-4475-8ccb-bbf380a0ed20") },
		{ "e2b4b868-4874-4254-b2c2-9e9983695c23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "e2b4b868-4874-4254-b2c2-9e9983695c23") },
		{ "2f0945cd-fefe-4964-839c-75413036309a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "2f0945cd-fefe-4964-839c-75413036309a") },
		{ "de54a0ab-1392-45c9-83b5-5c058254bfc8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "de54a0ab-1392-45c9-83b5-5c058254bfc8") },
		{ "f8172a17-ee31-45a4-b13a-7c8578c3328c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "f8172a17-ee31-45a4-b13a-7c8578c3328c") },
		{ "e6724654-492f-4731-ae8e-7d1cb7256ddd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "de62aa62-76bc-4f40-8dda-7c0d3eab3b81"}, { new Terminator(KeywordType.Integer), "a96775ea-398e-416c-803c-842482059a00"}, { new Terminator(KeywordType.Real), "815f5e68-35bd-4247-966e-8b3824c407dd"}, { new Terminator(KeywordType.Boolean), "23f04ad7-4e47-4ce1-b1c2-79ccc390b4c0"}, { new Terminator(KeywordType.Character), "90564982-90b1-4eaf-a3c5-467d173f730a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e6724654-492f-4731-ae8e-7d1cb7256ddd") },
		{ "8b1afb8d-01ba-4806-9586-24cdf08f2009", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "5ca2ccf3-6f2f-4a80-ba2a-8889aa818f63"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8b1afb8d-01ba-4806-9586-24cdf08f2009") },
		{ "e7d27dfa-4906-42dc-9f2c-13bce59cbe73", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "e7d27dfa-4906-42dc-9f2c-13bce59cbe73") },
		{ "b80a3b3f-ef3b-4da3-a18c-8dd45f2c8226", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "993860a7-b2bf-4d70-894a-51eef7ebafe6"}, { new NonTerminator(NonTerminatorType.Factor), "e000d176-1682-4847-8e70-ec982c4ae4fd"}, { Terminator.NumberTerminator, "725b2576-df41-4ca4-b8e5-62f9e4fdf931"}, { new NonTerminator(NonTerminatorType.Variable), "4c414e6c-c247-4826-b6d6-06609811edab"}, { new Terminator(DelimiterType.LeftParenthesis), "12a43d38-ab32-4511-a884-0fcbe38b57c6"}, { Terminator.IdentifierTerminator, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe"}, { new Terminator(KeywordType.Not), "8e0cd647-2a0c-4061-b5f1-e09beb035070"}, { new Terminator(OperatorType.Minus), "fc0dbccb-495a-4819-8333-766ee05b8fa0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b80a3b3f-ef3b-4da3-a18c-8dd45f2c8226") },
		{ "656702ea-dabc-404c-8c60-afcd411f04e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c39feeb3-3756-491e-bdf9-e1cde8d60efe"}, { Terminator.NumberTerminator, "725b2576-df41-4ca4-b8e5-62f9e4fdf931"}, { new NonTerminator(NonTerminatorType.Variable), "4c414e6c-c247-4826-b6d6-06609811edab"}, { new Terminator(DelimiterType.LeftParenthesis), "12a43d38-ab32-4511-a884-0fcbe38b57c6"}, { Terminator.IdentifierTerminator, "d1647a8b-675c-4c9d-81f1-e6a744eb5efe"}, { new Terminator(KeywordType.Not), "8e0cd647-2a0c-4061-b5f1-e09beb035070"}, { new Terminator(OperatorType.Minus), "fc0dbccb-495a-4819-8333-766ee05b8fa0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "656702ea-dabc-404c-8c60-afcd411f04e3") },
		{ "ce59caf4-1fe4-4ba0-ad72-a7310bb218db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "cf20ee8d-80f2-4623-a7c4-efa493b7520a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ce59caf4-1fe4-4ba0-ad72-a7310bb218db") },
		{ "897c1db1-c2c5-4ac8-8751-d4f5cc1cf135", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "aae7bcac-42df-4e70-a36b-3f7644d71702"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "897c1db1-c2c5-4ac8-8751-d4f5cc1cf135") },
		{ "a260110c-5289-4a9e-bedf-c13cfc5efabe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "a260110c-5289-4a9e-bedf-c13cfc5efabe") },
		{ "ab4b00e1-3b8c-487a-a0cf-426853bba2f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "878a8561-165d-4b85-b96d-e0717465153d"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ab4b00e1-3b8c-487a-a0cf-426853bba2f2") },
		{ "08928f2c-0dc0-4956-9597-9dbfdf8d04f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "08928f2c-0dc0-4956-9597-9dbfdf8d04f7") },
		{ "ec5068f6-63df-4ed0-a50f-38e25c74655c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "ec5068f6-63df-4ed0-a50f-38e25c74655c") },
		{ "83a11cfb-a850-4f3f-b9e1-7d93db67c90a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "83a11cfb-a850-4f3f-b9e1-7d93db67c90a") },
		{ "a5c43f99-cb84-4ab9-9565-65dc94316f3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a5c43f99-cb84-4ab9-9565-65dc94316f3d") },
		{ "028551ef-6ec6-4120-bb8a-dbad945234e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "fdca134c-72ed-4faa-92a5-636bff21f06c"}, { new NonTerminator(NonTerminatorType.Factor), "970e0165-64a5-4ec7-bae2-85181b442f90"}, { Terminator.NumberTerminator, "74f1d1b0-fcdb-43c9-935b-23f9465976e9"}, { new NonTerminator(NonTerminatorType.Variable), "835216be-d46c-4df4-84b4-d5785befeefa"}, { new Terminator(DelimiterType.LeftParenthesis), "9cd5687e-6fd5-4af7-8d99-b24d3988bf19"}, { Terminator.IdentifierTerminator, "d865fbc1-867d-4767-966b-c7d31b869fcb"}, { new Terminator(KeywordType.Not), "2f4d5cd4-5645-4832-801d-6e0d02c1cec6"}, { new Terminator(OperatorType.Minus), "a5118873-6eca-4d2c-b428-528624cff241"},}, new Dictionary<Terminator, ReduceInformation>{ }, "028551ef-6ec6-4120-bb8a-dbad945234e0") },
		{ "e3f07366-8e2b-4c79-bd53-16fc10a10be1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d7421a42-81cf-4cf4-b605-c6df847aaa70"}, { Terminator.NumberTerminator, "74f1d1b0-fcdb-43c9-935b-23f9465976e9"}, { new NonTerminator(NonTerminatorType.Variable), "835216be-d46c-4df4-84b4-d5785befeefa"}, { new Terminator(DelimiterType.LeftParenthesis), "9cd5687e-6fd5-4af7-8d99-b24d3988bf19"}, { Terminator.IdentifierTerminator, "d865fbc1-867d-4767-966b-c7d31b869fcb"}, { new Terminator(KeywordType.Not), "2f4d5cd4-5645-4832-801d-6e0d02c1cec6"}, { new Terminator(OperatorType.Minus), "a5118873-6eca-4d2c-b428-528624cff241"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3f07366-8e2b-4c79-bd53-16fc10a10be1") },
		{ "1d2c865b-9298-42f1-9cd5-0bab8dae1544", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "60ed1afa-9d9d-4e0c-b9ea-9dceffbdba02"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d2c865b-9298-42f1-9cd5-0bab8dae1544") },
		{ "1dc0db37-59f1-4f22-86f5-c6f575ad3a5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e3658a70-9cf8-4d5c-8fea-ba6611104645"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1dc0db37-59f1-4f22-86f5-c6f575ad3a5b") },
		{ "46ef772a-aba5-4efe-9628-f0a760243a89", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "46ef772a-aba5-4efe-9628-f0a760243a89") },
		{ "551be8c9-aad7-4461-8597-0dc4a3208189", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "43d5f743-dd7c-4afb-bede-20e96e531641"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "551be8c9-aad7-4461-8597-0dc4a3208189") },
		{ "3bf8b7a5-ec92-449a-af8f-872863834f09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3bf8b7a5-ec92-449a-af8f-872863834f09") },
		{ "6fcc126d-82e9-4602-8775-01cce0ecc602", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6fcc126d-82e9-4602-8775-01cce0ecc602") },
		{ "4b153968-6d82-4b65-9f8a-204c929b3cfe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "4b153968-6d82-4b65-9f8a-204c929b3cfe") },
		{ "04ce043b-fa3d-497d-9fb5-75617ab702c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "04ce043b-fa3d-497d-9fb5-75617ab702c7") },
		{ "c6c879a7-7627-49af-ab21-54ae0dfd8fc3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c65c8166-2a41-45d4-bb60-e6620261fb79"}, { new NonTerminator(NonTerminatorType.Factor), "e88da263-1b6c-40f3-b079-3d0fbeecb688"}, { Terminator.NumberTerminator, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7"}, { new NonTerminator(NonTerminatorType.Variable), "1abee674-834b-4b97-be87-2a3e0ae51d41"}, { new Terminator(DelimiterType.LeftParenthesis), "c65bb879-4e29-4e3c-a0a9-41f7a1c95045"}, { Terminator.IdentifierTerminator, "5970ac84-e42a-4e2f-af05-f2bf69693b8a"}, { new Terminator(KeywordType.Not), "b6a52b6c-2e7a-41cf-86e6-5261440d939e"}, { new Terminator(OperatorType.Minus), "27db342f-7a23-4075-b704-036f6de0ae54"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c6c879a7-7627-49af-ab21-54ae0dfd8fc3") },
		{ "4c6570dd-7342-4595-9139-6fcc9bafb635", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e354c946-7e73-4418-92f1-d5a2cedffe4b"}, { Terminator.NumberTerminator, "bb504128-cb1f-4fcd-b062-9a557fe4b3c7"}, { new NonTerminator(NonTerminatorType.Variable), "1abee674-834b-4b97-be87-2a3e0ae51d41"}, { new Terminator(DelimiterType.LeftParenthesis), "c65bb879-4e29-4e3c-a0a9-41f7a1c95045"}, { Terminator.IdentifierTerminator, "5970ac84-e42a-4e2f-af05-f2bf69693b8a"}, { new Terminator(KeywordType.Not), "b6a52b6c-2e7a-41cf-86e6-5261440d939e"}, { new Terminator(OperatorType.Minus), "27db342f-7a23-4075-b704-036f6de0ae54"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4c6570dd-7342-4595-9139-6fcc9bafb635") },
		{ "d11dfd34-a0a4-45c6-b41c-0fe59587ca00", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9014b33c-1b92-41f5-a525-5b9e4d3faf37"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d11dfd34-a0a4-45c6-b41c-0fe59587ca00") },
		{ "83b5acd7-84e8-4145-b4ad-ec2ff376ae5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0048537a-cb2f-40a8-b150-790a8f5a4179"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "83b5acd7-84e8-4145-b4ad-ec2ff376ae5f") },
		{ "aacab2d0-5282-4343-98fc-f64508a9e77a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "aacab2d0-5282-4343-98fc-f64508a9e77a") },
		{ "bc942d8a-fd96-41d8-8e50-5d1fba8d404c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "266b46b9-855b-4a63-87eb-d8ef526af082"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bc942d8a-fd96-41d8-8e50-5d1fba8d404c") },
		{ "7de0111a-dfeb-4b5b-aba0-8111d825f750", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7de0111a-dfeb-4b5b-aba0-8111d825f750") },
		{ "30f06511-3135-4567-971b-1340cd6617a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "30f06511-3135-4567-971b-1340cd6617a0") },
		{ "5e32f531-e627-4034-9560-2ade80ba4500", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "5e32f531-e627-4034-9560-2ade80ba4500") },
		{ "7e4b4e03-71b0-46da-853b-03b24807b0f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7e4b4e03-71b0-46da-853b-03b24807b0f0") },
		{ "9aebf022-347c-4065-8f28-47a67879c947", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "9aebf022-347c-4065-8f28-47a67879c947") },
		{ "00cde644-c96f-4f62-ab38-363354730ff8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "00cde644-c96f-4f62-ab38-363354730ff8") },
		{ "8b94a8a2-3347-4f1b-bcee-4b503291b55f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "8ce468e9-104b-4851-9bb7-aca06a032303"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "c93769d8-c6f0-4dd9-9e27-a4d513d57108"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "8b94a8a2-3347-4f1b-bcee-4b503291b55f") },
		{ "ce25b143-71c2-418e-8e4c-1e2f1a762a83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "adad5bc7-00bf-43e3-b066-8eb0fe2100c9"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ce25b143-71c2-418e-8e4c-1e2f1a762a83") },
		{ "2e69467b-75b2-43d4-ad3e-5362684cc4c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2e69467b-75b2-43d4-ad3e-5362684cc4c1") },
		{ "94c6ddbd-4c1b-41bb-9996-47da4a47b33f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f") },
		{ "679422f4-3e8d-47e6-902b-178b965aeb80", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "679422f4-3e8d-47e6-902b-178b965aeb80") },
		{ "962482cc-3670-494d-b0e8-8d59f5be9813", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ad8091d1-cb6b-40d1-8416-a312f0e73912"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "962482cc-3670-494d-b0e8-8d59f5be9813") },
		{ "9de51f12-31b4-4f9e-bd09-23c0c83f7b04", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "675646a7-4569-4722-9566-4d9e87893fe6"}, { new NonTerminator(NonTerminatorType.IdVarPart), "9a65a82a-8769-4d85-8e46-e69485dd49fd"}, { new Terminator(DelimiterType.LeftSquareBracket), "a765252c-43d8-4125-9272-943bd0bf011b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04") },
		{ "ad6ffd4e-81a8-4d7e-b497-a85d875dc798", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "96c72e58-0ae2-4172-adc6-c61eeb99d635"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad6ffd4e-81a8-4d7e-b497-a85d875dc798") },
		{ "be94c4f8-3bed-4fb0-9792-34ca6155b56f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "10c5c2d4-6ee4-458d-bed7-046ad3237820"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be94c4f8-3bed-4fb0-9792-34ca6155b56f") },
		{ "d451b2b0-da56-414e-b7e1-c71cba23f495", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "d451b2b0-da56-414e-b7e1-c71cba23f495") },
		{ "5531e016-2693-447a-8395-b7a188ec650f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "709ace44-cc95-4192-9285-1bbab6f3669d"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5531e016-2693-447a-8395-b7a188ec650f") },
		{ "502ad32f-85c0-4328-aa69-199378bf5013", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "cb31c0e4-1ef3-409e-8e40-0e51ab6f6e1a"}, { new NonTerminator(NonTerminatorType.Variable), "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36"}, { Terminator.IdentifierTerminator, "d74ee885-45f2-4595-a66a-53e932d8391f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b8794192-f4c9-4a1b-916f-bfaf49f43208"}, { new Terminator(KeywordType.If), "f6810f84-1a46-4894-8c44-c2f8068e5c2f"}, { new Terminator(KeywordType.For), "954e1e88-c806-45af-84bf-a00026c87a9c"}, { new Terminator(KeywordType.Begin), "52ecab85-fa9a-45b6-942d-b6de60d7087c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "502ad32f-85c0-4328-aa69-199378bf5013") },
		{ "8269c0b3-0051-4e2c-b2c6-2485ab13268d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "94cf67c5-0927-4baa-86ff-c22a56a8dca0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "9c417c34-245c-4d71-ac9b-a095bc19f366"}, { new NonTerminator(NonTerminatorType.Term), "9b515565-da09-46ce-aa9d-2a7969ad537c"}, { new NonTerminator(NonTerminatorType.Factor), "ae5fa63b-fa83-4af1-a392-77c6c28fc8c9"}, { Terminator.NumberTerminator, "96f94b1f-c66e-4a9b-abdc-2c4e2bf9f557"}, { new NonTerminator(NonTerminatorType.Variable), "44175282-e122-4250-b19e-9ab68843ffcf"}, { new Terminator(DelimiterType.LeftParenthesis), "36d93a6b-7c19-4a6a-ada1-5e936f3ef57f"}, { Terminator.IdentifierTerminator, "6a5f55c8-d8a8-41ed-bf3e-bfce19cc024d"}, { new Terminator(KeywordType.Not), "3668d476-b4f2-445a-8830-e21aafcd18fc"}, { new Terminator(OperatorType.Minus), "5fbeab4a-5be6-458b-8366-e945e9d1561d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8269c0b3-0051-4e2c-b2c6-2485ab13268d") },
		{ "73113c3f-17d9-4344-9a39-cecc8b70d2ac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "73113c3f-17d9-4344-9a39-cecc8b70d2ac") },
		{ "b5687c72-830b-429d-a30c-76148d840e22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "ce325b26-efe9-45e0-ab22-7c9e1441e089"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b5687c72-830b-429d-a30c-76148d840e22") },
		{ "e44bd5be-d5bc-46b3-8078-e291cc91a7df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "e44bd5be-d5bc-46b3-8078-e291cc91a7df") },
		{ "78f99f90-ff30-40cd-83df-f7bc1950330a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "78f99f90-ff30-40cd-83df-f7bc1950330a") },
		{ "62917d15-21c0-4112-afdb-f18a1d4bcb5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "3b1c62b1-21c6-4775-b743-6f1703c883e2"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "62917d15-21c0-4112-afdb-f18a1d4bcb5b") },
		{ "cf7dc292-45e2-4322-9730-55e10494526f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "e5834a9b-67a6-4752-b303-b0e55419dc7c"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cf7dc292-45e2-4322-9730-55e10494526f") },
		{ "4b3f9459-fe04-461b-9ed0-2ec69678345a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "0271fcc4-4122-4e13-b5e7-aafec96c9d5d"}, { new NonTerminator(NonTerminatorType.Factor), "d3912847-97a1-465d-9cd7-6dbba61cfbd7"}, { Terminator.NumberTerminator, "3c073275-f254-4aca-987c-1934f4682c0c"}, { new NonTerminator(NonTerminatorType.Variable), "b1a86855-464b-4b79-8cc2-bbfb5e375f74"}, { new Terminator(DelimiterType.LeftParenthesis), "966e3e24-a183-4c9b-83ef-09b1f41dd824"}, { Terminator.IdentifierTerminator, "3616b1e4-3979-4495-8908-afa295f8f030"}, { new Terminator(KeywordType.Not), "2572d6b4-5b66-47c2-bcaa-f40003515d7a"}, { new Terminator(OperatorType.Minus), "70dc3b23-0ce9-4ded-a0b3-1e683d782b30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b3f9459-fe04-461b-9ed0-2ec69678345a") },
		{ "3371f493-d8d4-441d-8e9d-64b1a1a3ffeb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a0314d07-c789-436f-84bd-8acb969ab7ea"}, { Terminator.NumberTerminator, "3c073275-f254-4aca-987c-1934f4682c0c"}, { new NonTerminator(NonTerminatorType.Variable), "b1a86855-464b-4b79-8cc2-bbfb5e375f74"}, { new Terminator(DelimiterType.LeftParenthesis), "966e3e24-a183-4c9b-83ef-09b1f41dd824"}, { Terminator.IdentifierTerminator, "3616b1e4-3979-4495-8908-afa295f8f030"}, { new Terminator(KeywordType.Not), "2572d6b4-5b66-47c2-bcaa-f40003515d7a"}, { new Terminator(OperatorType.Minus), "70dc3b23-0ce9-4ded-a0b3-1e683d782b30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3371f493-d8d4-441d-8e9d-64b1a1a3ffeb") },
		{ "d0445134-77cc-4724-8954-cf34a23129e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a93a05aa-b5e9-409c-b733-7ad12a00089e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d0445134-77cc-4724-8954-cf34a23129e6") },
		{ "d530f6b4-3f5e-4b3f-8856-dabd17a50828", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "14ddad8e-fd7e-4bdb-a150-be9ac8918e66"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d530f6b4-3f5e-4b3f-8856-dabd17a50828") },
		{ "76e96605-607d-4447-9b54-bf5c6575760f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "76e96605-607d-4447-9b54-bf5c6575760f") },
		{ "6fcb77eb-8903-41e2-9a38-ea764fe684e3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "80794a1a-c61d-4459-a676-1347041fd374"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6fcb77eb-8903-41e2-9a38-ea764fe684e3") },
		{ "3cf42241-f0c2-4108-9110-baded553ef25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3cf42241-f0c2-4108-9110-baded553ef25") },
		{ "199684b4-cbcc-49de-95eb-2c63a9073a88", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "199684b4-cbcc-49de-95eb-2c63a9073a88") },
		{ "e3c61076-2a64-4b9b-918a-6e32acd31dc5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e3c61076-2a64-4b9b-918a-6e32acd31dc5") },
		{ "8ed3f13d-a162-470e-886a-684b814d96eb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8ed3f13d-a162-470e-886a-684b814d96eb") },
		{ "bd5d1ff4-dece-487d-b6ec-2784faed1742", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "f8c5142d-185b-4c94-95cf-a4a67ee98664"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bd5d1ff4-dece-487d-b6ec-2784faed1742") },
		{ "cb5620f4-43e3-4e20-a089-373f30c4d741", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "5ea1392e-9617-44bd-a337-85520494523b"}, { new Terminator(OperatorType.Equal), "0e7d5c37-773d-432f-a8cf-784bb6b0efe3"}, { new Terminator(OperatorType.NotEqual), "be609488-c42f-47f8-a1fd-f2d444c93bc8"}, { new Terminator(OperatorType.Less), "44f4764c-0b4f-4c28-864a-1001907823d5"}, { new Terminator(OperatorType.LessEqual), "07064b0f-4530-4a28-8fa2-42b3fdb533b1"}, { new Terminator(OperatorType.Greater), "75cf3ae1-b17b-4a65-a659-706f30e30068"}, { new Terminator(OperatorType.GreaterEqual), "177eed3f-ff83-48f0-88e3-46d396939896"}, { new NonTerminator(NonTerminatorType.AddOperator), "4911458f-a663-4447-b29a-fb26e47da009"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "cb5620f4-43e3-4e20-a089-373f30c4d741") },
		{ "460855ab-487d-4f1d-90e4-d922f0bd6520", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "eac9c4a3-bbed-49e9-83e4-742f9a804fdc"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "460855ab-487d-4f1d-90e4-d922f0bd6520") },
		{ "3a8d241f-315a-440c-a585-2cf6dee76653", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "3a8d241f-315a-440c-a585-2cf6dee76653") },
		{ "651037b4-7068-417b-8a7b-9b69bbf4c6df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "651037b4-7068-417b-8a7b-9b69bbf4c6df") },
		{ "bafc1f87-7778-4ac0-aa45-59d7d5226923", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bafc1f87-7778-4ac0-aa45-59d7d5226923") },
		{ "caa330bc-cc14-4679-bafb-47b37c5f4ae0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "487ed6fa-80fc-4a9d-af0b-9da363e6b3e8"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "caa330bc-cc14-4679-bafb-47b37c5f4ae0") },
		{ "42943c17-2772-4672-826f-ba44db9d32cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "a40e50e3-7f0e-4355-9e13-7d1a2431c9a7"}, { new NonTerminator(NonTerminatorType.IdVarPart), "373f03a2-54ce-4ebf-9f24-a0bbbb3d76c6"}, { new Terminator(DelimiterType.LeftSquareBracket), "fc161faa-09e7-4299-a6e3-57816e414ecf"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "42943c17-2772-4672-826f-ba44db9d32cd") },
		{ "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9b02434e-7aee-41fd-a87c-77ebf3e27c96"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2") },
		{ "df993462-ad4d-456e-8079-06b377073a3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c0429956-50ef-4c29-9ed9-5576582a4568"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df993462-ad4d-456e-8079-06b377073a3e") },
		{ "f58d81b4-32b7-4db6-b7e2-d7703194cd73", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "22af6167-8856-4410-821c-0b0764c0c3af"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "f58d81b4-32b7-4db6-b7e2-d7703194cd73") },
		{ "2826d44b-91bb-4f74-b2ee-d25438624989", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c2613bbd-7b4a-4ffc-910b-4669b7bc7b0d"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2826d44b-91bb-4f74-b2ee-d25438624989") },
		{ "950fb149-1176-4711-8afb-9a7ebf5cbb8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "950fb149-1176-4711-8afb-9a7ebf5cbb8d") },
		{ "579d451b-936b-48a4-b3fc-09a46555f5cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "579d451b-936b-48a4-b3fc-09a46555f5cd") },
		{ "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf") },
		{ "ba3fc134-6a9a-4554-aa1c-6acba919674e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "42b219bf-b528-4dc9-9226-ad1ae3ac382b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ba3fc134-6a9a-4554-aa1c-6acba919674e") },
		{ "fedef25c-c4e4-436a-9c87-f345407b58ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "84b44fc2-87d4-4d52-8c49-be962349a173"}, { new NonTerminator(NonTerminatorType.IdVarPart), "f67cd709-f502-492a-a321-c9bf26a7f405"}, { new Terminator(DelimiterType.LeftSquareBracket), "40ac09fe-90df-43b5-947d-980faa8f8d77"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "fedef25c-c4e4-436a-9c87-f345407b58ab") },
		{ "06dd1dba-097b-46c1-8406-acf688137867", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c2d33029-5a0b-450c-bb41-767fd722d0ed"}, { Terminator.NumberTerminator, "579d451b-936b-48a4-b3fc-09a46555f5cd"}, { new NonTerminator(NonTerminatorType.Variable), "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf"}, { new Terminator(DelimiterType.LeftParenthesis), "ba3fc134-6a9a-4554-aa1c-6acba919674e"}, { Terminator.IdentifierTerminator, "fedef25c-c4e4-436a-9c87-f345407b58ab"}, { new Terminator(KeywordType.Not), "06dd1dba-097b-46c1-8406-acf688137867"}, { new Terminator(OperatorType.Minus), "ca6eb5a5-9781-4977-97bc-daba801a61fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "06dd1dba-097b-46c1-8406-acf688137867") },
		{ "ca6eb5a5-9781-4977-97bc-daba801a61fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b974b29d-4798-4db8-b2d6-f42ebbe67d6d"}, { Terminator.NumberTerminator, "579d451b-936b-48a4-b3fc-09a46555f5cd"}, { new NonTerminator(NonTerminatorType.Variable), "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf"}, { new Terminator(DelimiterType.LeftParenthesis), "ba3fc134-6a9a-4554-aa1c-6acba919674e"}, { Terminator.IdentifierTerminator, "fedef25c-c4e4-436a-9c87-f345407b58ab"}, { new Terminator(KeywordType.Not), "06dd1dba-097b-46c1-8406-acf688137867"}, { new Terminator(OperatorType.Minus), "ca6eb5a5-9781-4977-97bc-daba801a61fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ca6eb5a5-9781-4977-97bc-daba801a61fa") },
		{ "d93dd101-59ac-4533-8f47-09910cfe1336", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "244157d5-9074-40a6-8598-3a1c9f695f91"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d93dd101-59ac-4533-8f47-09910cfe1336") },
		{ "f956d88a-3fae-4b42-af7b-3cf613e05c32", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "f956d88a-3fae-4b42-af7b-3cf613e05c32") },
		{ "a061df0a-5c2b-4c7e-8ce0-e71f67da8415", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a061df0a-5c2b-4c7e-8ce0-e71f67da8415") },
		{ "4d4f158f-9dac-4aed-ac99-af254890ac26", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "bdf51351-1f75-4a10-befb-88d0a67113ad"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4d4f158f-9dac-4aed-ac99-af254890ac26") },
		{ "e79929ab-9cf5-43ae-8fa7-2031298c3d9b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "afec78a9-2219-4557-80da-d2685c565aa4"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e79929ab-9cf5-43ae-8fa7-2031298c3d9b") },
		{ "b7c2ead4-246b-4eee-9e9a-a5e1b8bbe083", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "b7c2ead4-246b-4eee-9e9a-a5e1b8bbe083") },
		{ "de62aa62-76bc-4f40-8dda-7c0d3eab3b81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "de62aa62-76bc-4f40-8dda-7c0d3eab3b81") },
		{ "5ca2ccf3-6f2f-4a80-ba2a-8889aa818f63", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "3ccbc319-8a71-4e8b-b56e-510e819093f0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5ca2ccf3-6f2f-4a80-ba2a-8889aa818f63") },
		{ "993860a7-b2bf-4d70-894a-51eef7ebafe6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "656702ea-dabc-404c-8c60-afcd411f04e3"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "993860a7-b2bf-4d70-894a-51eef7ebafe6") },
		{ "c39feeb3-3756-491e-bdf9-e1cde8d60efe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "c39feeb3-3756-491e-bdf9-e1cde8d60efe") },
		{ "cf20ee8d-80f2-4623-a7c4-efa493b7520a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "cf20ee8d-80f2-4623-a7c4-efa493b7520a") },
		{ "aae7bcac-42df-4e70-a36b-3f7644d71702", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "058ae5c0-aa69-4d5f-91b6-90728e3bf3c6"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "aae7bcac-42df-4e70-a36b-3f7644d71702") },
		{ "878a8561-165d-4b85-b96d-e0717465153d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "f15eac32-d9bf-4dfd-add3-ae54daab03d9"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "878a8561-165d-4b85-b96d-e0717465153d") },
		{ "fdca134c-72ed-4faa-92a5-636bff21f06c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "e3f07366-8e2b-4c79-bd53-16fc10a10be1"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fdca134c-72ed-4faa-92a5-636bff21f06c") },
		{ "d7421a42-81cf-4cf4-b605-c6df847aaa70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "d7421a42-81cf-4cf4-b605-c6df847aaa70") },
		{ "60ed1afa-9d9d-4e0c-b9ea-9dceffbdba02", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "60ed1afa-9d9d-4e0c-b9ea-9dceffbdba02") },
		{ "e3658a70-9cf8-4d5c-8fea-ba6611104645", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "37039e09-4b7d-4359-8f89-64e2b0721f16"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e3658a70-9cf8-4d5c-8fea-ba6611104645") },
		{ "43d5f743-dd7c-4afb-bede-20e96e531641", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "da634c47-0f51-4fb3-97f2-b06c7b23bed3"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "43d5f743-dd7c-4afb-bede-20e96e531641") },
		{ "c65c8166-2a41-45d4-bb60-e6620261fb79", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "4c6570dd-7342-4595-9139-6fcc9bafb635"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c65c8166-2a41-45d4-bb60-e6620261fb79") },
		{ "e354c946-7e73-4418-92f1-d5a2cedffe4b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "e354c946-7e73-4418-92f1-d5a2cedffe4b") },
		{ "9014b33c-1b92-41f5-a525-5b9e4d3faf37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9014b33c-1b92-41f5-a525-5b9e4d3faf37") },
		{ "0048537a-cb2f-40a8-b150-790a8f5a4179", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8a3223d0-2ca6-4dc8-acf8-47e73d6474e5"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0048537a-cb2f-40a8-b150-790a8f5a4179") },
		{ "266b46b9-855b-4a63-87eb-d8ef526af082", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "13f87663-f814-4e1b-a255-8d99f370c11f"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "266b46b9-855b-4a63-87eb-d8ef526af082") },
		{ "8ce468e9-104b-4851-9bb7-aca06a032303", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "bf421c0b-f715-4a83-b21c-aa4262db28a9"}, { new NonTerminator(NonTerminatorType.Term), "a2eed3f8-3f53-4c52-aa9a-d6e35bfc4921"}, { new NonTerminator(NonTerminatorType.Factor), "26001bff-97ea-41b3-a426-f1a0b970d340"}, { Terminator.NumberTerminator, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47"}, { new NonTerminator(NonTerminatorType.Variable), "62195e40-a979-4524-9d19-3768c9af6073"}, { new Terminator(DelimiterType.LeftParenthesis), "20e7d417-07fc-4b80-a45d-01089114c58f"}, { Terminator.IdentifierTerminator, "fd68001e-0c67-4d09-820d-b22d33a6c89d"}, { new Terminator(KeywordType.Not), "2ef67070-7169-4318-8725-6117f46ed3fe"}, { new Terminator(OperatorType.Minus), "06cefe9b-a60e-4214-807e-2ddbf587c264"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8ce468e9-104b-4851-9bb7-aca06a032303") },
		{ "c93769d8-c6f0-4dd9-9e27-a4d513d57108", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "3c007ba8-d36c-479b-a105-48a7e14fd71d"}, { new NonTerminator(NonTerminatorType.Factor), "2e69467b-75b2-43d4-ad3e-5362684cc4c1"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c93769d8-c6f0-4dd9-9e27-a4d513d57108") },
		{ "adad5bc7-00bf-43e3-b066-8eb0fe2100c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9771e833-20be-49af-9005-c525c1c84552"}, { Terminator.NumberTerminator, "94c6ddbd-4c1b-41bb-9996-47da4a47b33f"}, { new NonTerminator(NonTerminatorType.Variable), "679422f4-3e8d-47e6-902b-178b965aeb80"}, { new Terminator(DelimiterType.LeftParenthesis), "962482cc-3670-494d-b0e8-8d59f5be9813"}, { Terminator.IdentifierTerminator, "9de51f12-31b4-4f9e-bd09-23c0c83f7b04"}, { new Terminator(KeywordType.Not), "ad6ffd4e-81a8-4d7e-b497-a85d875dc798"}, { new Terminator(OperatorType.Minus), "be94c4f8-3bed-4fb0-9792-34ca6155b56f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "adad5bc7-00bf-43e3-b066-8eb0fe2100c9") },
		{ "ad8091d1-cb6b-40d1-8416-a312f0e73912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b6e11ccc-7703-4e6d-a769-a81c4a7ecab7"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad8091d1-cb6b-40d1-8416-a312f0e73912") },
		{ "675646a7-4569-4722-9566-4d9e87893fe6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9a45007f-579e-48b8-806a-6798cdd3dc27"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "675646a7-4569-4722-9566-4d9e87893fe6") },
		{ "9a65a82a-8769-4d85-8e46-e69485dd49fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "9a65a82a-8769-4d85-8e46-e69485dd49fd") },
		{ "a765252c-43d8-4125-9272-943bd0bf011b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "26de26a3-e614-4d03-97da-3dca8f98d636"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a765252c-43d8-4125-9272-943bd0bf011b") },
		{ "96c72e58-0ae2-4172-adc6-c61eeb99d635", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "96c72e58-0ae2-4172-adc6-c61eeb99d635") },
		{ "10c5c2d4-6ee4-458d-bed7-046ad3237820", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "10c5c2d4-6ee4-458d-bed7-046ad3237820") },
		{ "709ace44-cc95-4192-9285-1bbab6f3669d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "709ace44-cc95-4192-9285-1bbab6f3669d") },
		{ "cb31c0e4-1ef3-409e-8e40-0e51ab6f6e1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "fc822cb9-5467-4fac-9501-b93949cbf700"}, { new Terminator(KeywordType.Else), "090cfe19-bbea-46e0-a83c-8bfcc95dcdd9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "cb31c0e4-1ef3-409e-8e40-0e51ab6f6e1a") },
		{ "94cf67c5-0927-4baa-86ff-c22a56a8dca0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "8b35d723-e8cc-4aeb-b653-0a9bfd41bb7e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "94cf67c5-0927-4baa-86ff-c22a56a8dca0") },
		{ "3b1c62b1-21c6-4775-b743-6f1703c883e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "3b1c62b1-21c6-4775-b743-6f1703c883e2") },
		{ "e5834a9b-67a6-4752-b303-b0e55419dc7c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e5834a9b-67a6-4752-b303-b0e55419dc7c") },
		{ "0271fcc4-4122-4e13-b5e7-aafec96c9d5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "3371f493-d8d4-441d-8e9d-64b1a1a3ffeb"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0271fcc4-4122-4e13-b5e7-aafec96c9d5d") },
		{ "a0314d07-c789-436f-84bd-8acb969ab7ea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "a0314d07-c789-436f-84bd-8acb969ab7ea") },
		{ "a93a05aa-b5e9-409c-b733-7ad12a00089e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a93a05aa-b5e9-409c-b733-7ad12a00089e") },
		{ "14ddad8e-fd7e-4bdb-a150-be9ac8918e66", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "51e258a3-2e92-4139-b537-69d46b634fbc"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "14ddad8e-fd7e-4bdb-a150-be9ac8918e66") },
		{ "80794a1a-c61d-4459-a676-1347041fd374", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d8e226c9-63ad-4ee2-9fc3-d34f2019eb62"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80794a1a-c61d-4459-a676-1347041fd374") },
		{ "f8c5142d-185b-4c94-95cf-a4a67ee98664", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "823e144a-de85-4656-bd9d-9fa8da74fd5c"}, { new NonTerminator(NonTerminatorType.Variable), "340c5627-5962-46b7-982e-a7f4da6071ef"}, { Terminator.IdentifierTerminator, "15a67b60-7589-4ccd-b0a1-e0e05e419a76"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "6f286d93-70c5-447e-96f3-55dd578788bc"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "baa01d82-c80e-429e-948f-e1c69ad8c4c4"}, { new Terminator(KeywordType.If), "b43df555-7769-42ab-b698-430d230f6a3e"}, { new Terminator(KeywordType.For), "b1fb6f3f-3e47-48f5-9169-077bd24dfc5f"}, { new Terminator(KeywordType.Begin), "6e50af1f-a22a-432f-80cd-9c938375d882"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "f8c5142d-185b-4c94-95cf-a4a67ee98664") },
		{ "5ea1392e-9617-44bd-a337-85520494523b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "b6c55903-13af-4bf0-965f-27c62ebb3a54"}, { new NonTerminator(NonTerminatorType.Term), "9e489b6e-aebf-4bf0-b95a-1e8b4c30d0e2"}, { new NonTerminator(NonTerminatorType.Factor), "364a7211-dfcd-4426-985b-e5eef6b8b3d1"}, { Terminator.NumberTerminator, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8"}, { new NonTerminator(NonTerminatorType.Variable), "23db53df-9747-4c31-a859-5da5e46b654a"}, { new Terminator(DelimiterType.LeftParenthesis), "9653fd90-788b-4082-a9c5-5bf9d878771a"}, { Terminator.IdentifierTerminator, "d8a5a803-55a0-48d9-9e26-cfa20c699613"}, { new Terminator(KeywordType.Not), "be7d2ff7-c402-48b9-8aeb-1e78a5e56089"}, { new Terminator(OperatorType.Minus), "17272433-e4b7-42e0-9d3a-d5c5e24057cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5ea1392e-9617-44bd-a337-85520494523b") },
		{ "4911458f-a663-4447-b29a-fb26e47da009", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "a581004e-ba9a-4061-9bf6-d2e738c290bf"}, { new NonTerminator(NonTerminatorType.Factor), "3a8d241f-315a-440c-a585-2cf6dee76653"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4911458f-a663-4447-b29a-fb26e47da009") },
		{ "eac9c4a3-bbed-49e9-83e4-742f9a804fdc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d16fa4ca-95e9-465c-8022-3ac13b82a5b4"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eac9c4a3-bbed-49e9-83e4-742f9a804fdc") },
		{ "487ed6fa-80fc-4a9d-af0b-9da363e6b3e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a0903bba-0ae6-4034-baad-e95606db659c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "487ed6fa-80fc-4a9d-af0b-9da363e6b3e8") },
		{ "a40e50e3-7f0e-4355-9e13-7d1a2431c9a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d1d964d3-e97c-4fd0-828b-1fd0c920ea1a"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a40e50e3-7f0e-4355-9e13-7d1a2431c9a7") },
		{ "373f03a2-54ce-4ebf-9f24-a0bbbb3d76c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "373f03a2-54ce-4ebf-9f24-a0bbbb3d76c6") },
		{ "fc161faa-09e7-4299-a6e3-57816e414ecf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ff5e7c0c-b3e5-4297-8526-321729adb384"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fc161faa-09e7-4299-a6e3-57816e414ecf") },
		{ "9b02434e-7aee-41fd-a87c-77ebf3e27c96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9b02434e-7aee-41fd-a87c-77ebf3e27c96") },
		{ "c0429956-50ef-4c29-9ed9-5576582a4568", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c0429956-50ef-4c29-9ed9-5576582a4568") },
		{ "22af6167-8856-4410-821c-0b0764c0c3af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "ec2ad6f2-68db-4a08-9610-fb81f872429d"}, { new NonTerminator(NonTerminatorType.Factor), "950fb149-1176-4711-8afb-9a7ebf5cbb8d"}, { Terminator.NumberTerminator, "579d451b-936b-48a4-b3fc-09a46555f5cd"}, { new NonTerminator(NonTerminatorType.Variable), "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf"}, { new Terminator(DelimiterType.LeftParenthesis), "ba3fc134-6a9a-4554-aa1c-6acba919674e"}, { Terminator.IdentifierTerminator, "fedef25c-c4e4-436a-9c87-f345407b58ab"}, { new Terminator(KeywordType.Not), "06dd1dba-097b-46c1-8406-acf688137867"}, { new Terminator(OperatorType.Minus), "ca6eb5a5-9781-4977-97bc-daba801a61fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "22af6167-8856-4410-821c-0b0764c0c3af") },
		{ "c2613bbd-7b4a-4ffc-910b-4669b7bc7b0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e7075224-4686-443d-a873-8b465777ae51"}, { Terminator.NumberTerminator, "579d451b-936b-48a4-b3fc-09a46555f5cd"}, { new NonTerminator(NonTerminatorType.Variable), "e46c1b0a-90a1-4123-97a5-9b9ba6dc49bf"}, { new Terminator(DelimiterType.LeftParenthesis), "ba3fc134-6a9a-4554-aa1c-6acba919674e"}, { Terminator.IdentifierTerminator, "fedef25c-c4e4-436a-9c87-f345407b58ab"}, { new Terminator(KeywordType.Not), "06dd1dba-097b-46c1-8406-acf688137867"}, { new Terminator(OperatorType.Minus), "ca6eb5a5-9781-4977-97bc-daba801a61fa"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2613bbd-7b4a-4ffc-910b-4669b7bc7b0d") },
		{ "42b219bf-b528-4dc9-9226-ad1ae3ac382b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4f22a56b-267b-4196-9d9a-1118c90e7d45"},}, new Dictionary<Terminator, ReduceInformation>{ }, "42b219bf-b528-4dc9-9226-ad1ae3ac382b") },
		{ "84b44fc2-87d4-4d52-8c49-be962349a173", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "df953bf7-7d51-48d1-bd2c-115ddae453ef"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "84b44fc2-87d4-4d52-8c49-be962349a173") },
		{ "f67cd709-f502-492a-a321-c9bf26a7f405", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "f67cd709-f502-492a-a321-c9bf26a7f405") },
		{ "40ac09fe-90df-43b5-947d-980faa8f8d77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "977d195d-6a91-4247-be5d-8b30fe200b1b"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "40ac09fe-90df-43b5-947d-980faa8f8d77") },
		{ "c2d33029-5a0b-450c-bb41-767fd722d0ed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c2d33029-5a0b-450c-bb41-767fd722d0ed") },
		{ "b974b29d-4798-4db8-b2d6-f42ebbe67d6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b974b29d-4798-4db8-b2d6-f42ebbe67d6d") },
		{ "bdf51351-1f75-4a10-befb-88d0a67113ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "bdf51351-1f75-4a10-befb-88d0a67113ad") },
		{ "afec78a9-2219-4557-80da-d2685c565aa4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "afec78a9-2219-4557-80da-d2685c565aa4") },
		{ "3ccbc319-8a71-4e8b-b56e-510e819093f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "3ccbc319-8a71-4e8b-b56e-510e819093f0") },
		{ "058ae5c0-aa69-4d5f-91b6-90728e3bf3c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "058ae5c0-aa69-4d5f-91b6-90728e3bf3c6") },
		{ "f15eac32-d9bf-4dfd-add3-ae54daab03d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "f15eac32-d9bf-4dfd-add3-ae54daab03d9") },
		{ "37039e09-4b7d-4359-8f89-64e2b0721f16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "37039e09-4b7d-4359-8f89-64e2b0721f16") },
		{ "da634c47-0f51-4fb3-97f2-b06c7b23bed3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "da634c47-0f51-4fb3-97f2-b06c7b23bed3") },
		{ "8a3223d0-2ca6-4dc8-acf8-47e73d6474e5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "8a3223d0-2ca6-4dc8-acf8-47e73d6474e5") },
		{ "13f87663-f814-4e1b-a255-8d99f370c11f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "13f87663-f814-4e1b-a255-8d99f370c11f") },
		{ "bf421c0b-f715-4a83-b21c-aa4262db28a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "451ea151-ba9e-45d8-8e79-c85a60d9b8e0"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "bf421c0b-f715-4a83-b21c-aa4262db28a9") },
		{ "a2eed3f8-3f53-4c52-aa9a-d6e35bfc4921", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c87c5ade-7537-424a-874d-ff8e2315da69"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a2eed3f8-3f53-4c52-aa9a-d6e35bfc4921") },
		{ "26001bff-97ea-41b3-a426-f1a0b970d340", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "26001bff-97ea-41b3-a426-f1a0b970d340") },
		{ "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47") },
		{ "62195e40-a979-4524-9d19-3768c9af6073", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "62195e40-a979-4524-9d19-3768c9af6073") },
		{ "20e7d417-07fc-4b80-a45d-01089114c58f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "b837453e-6734-4ed9-b83c-75008a9131c4"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "20e7d417-07fc-4b80-a45d-01089114c58f") },
		{ "fd68001e-0c67-4d09-820d-b22d33a6c89d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "10791e8d-ebd0-4ffa-8418-3724332af02d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "9ae20f4e-fb27-4c38-824b-c2c6ceb93582"}, { new Terminator(DelimiterType.LeftSquareBracket), "3823e6d1-d514-412d-8d64-77ee7e6c7541"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "fd68001e-0c67-4d09-820d-b22d33a6c89d") },
		{ "2ef67070-7169-4318-8725-6117f46ed3fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "655d58c6-903e-4eb2-a39a-083ad64393de"}, { Terminator.NumberTerminator, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47"}, { new NonTerminator(NonTerminatorType.Variable), "62195e40-a979-4524-9d19-3768c9af6073"}, { new Terminator(DelimiterType.LeftParenthesis), "20e7d417-07fc-4b80-a45d-01089114c58f"}, { Terminator.IdentifierTerminator, "fd68001e-0c67-4d09-820d-b22d33a6c89d"}, { new Terminator(KeywordType.Not), "2ef67070-7169-4318-8725-6117f46ed3fe"}, { new Terminator(OperatorType.Minus), "06cefe9b-a60e-4214-807e-2ddbf587c264"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2ef67070-7169-4318-8725-6117f46ed3fe") },
		{ "06cefe9b-a60e-4214-807e-2ddbf587c264", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "26c3389e-83b1-4edc-92f8-ff364328ec77"}, { Terminator.NumberTerminator, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47"}, { new NonTerminator(NonTerminatorType.Variable), "62195e40-a979-4524-9d19-3768c9af6073"}, { new Terminator(DelimiterType.LeftParenthesis), "20e7d417-07fc-4b80-a45d-01089114c58f"}, { Terminator.IdentifierTerminator, "fd68001e-0c67-4d09-820d-b22d33a6c89d"}, { new Terminator(KeywordType.Not), "2ef67070-7169-4318-8725-6117f46ed3fe"}, { new Terminator(OperatorType.Minus), "06cefe9b-a60e-4214-807e-2ddbf587c264"},}, new Dictionary<Terminator, ReduceInformation>{ }, "06cefe9b-a60e-4214-807e-2ddbf587c264") },
		{ "3c007ba8-d36c-479b-a105-48a7e14fd71d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "adad5bc7-00bf-43e3-b066-8eb0fe2100c9"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "3c007ba8-d36c-479b-a105-48a7e14fd71d") },
		{ "9771e833-20be-49af-9005-c525c1c84552", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "9771e833-20be-49af-9005-c525c1c84552") },
		{ "b6e11ccc-7703-4e6d-a769-a81c4a7ecab7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b6e11ccc-7703-4e6d-a769-a81c4a7ecab7") },
		{ "9a45007f-579e-48b8-806a-6798cdd3dc27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "751a3cbb-06d3-41fb-92c1-5783ed65f60f"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9a45007f-579e-48b8-806a-6798cdd3dc27") },
		{ "26de26a3-e614-4d03-97da-3dca8f98d636", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "e5247481-e0a9-4897-9675-c22910c5c5b5"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "26de26a3-e614-4d03-97da-3dca8f98d636") },
		{ "fc822cb9-5467-4fac-9501-b93949cbf700", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "fc822cb9-5467-4fac-9501-b93949cbf700") },
		{ "090cfe19-bbea-46e0-a83c-8bfcc95dcdd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "36630a09-3be1-4c05-af14-281bc3a4ea29"}, { new NonTerminator(NonTerminatorType.Variable), "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36"}, { Terminator.IdentifierTerminator, "d74ee885-45f2-4595-a66a-53e932d8391f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b8794192-f4c9-4a1b-916f-bfaf49f43208"}, { new Terminator(KeywordType.If), "f6810f84-1a46-4894-8c44-c2f8068e5c2f"}, { new Terminator(KeywordType.For), "954e1e88-c806-45af-84bf-a00026c87a9c"}, { new Terminator(KeywordType.Begin), "52ecab85-fa9a-45b6-942d-b6de60d7087c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "090cfe19-bbea-46e0-a83c-8bfcc95dcdd9") },
		{ "8b35d723-e8cc-4aeb-b653-0a9bfd41bb7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6339d066-ac25-4885-91d1-a857e19c4e0d"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "cb5620f4-43e3-4e20-a089-373f30c4d741"}, { new NonTerminator(NonTerminatorType.Term), "460855ab-487d-4f1d-90e4-d922f0bd6520"}, { new NonTerminator(NonTerminatorType.Factor), "3a8d241f-315a-440c-a585-2cf6dee76653"}, { Terminator.NumberTerminator, "651037b4-7068-417b-8a7b-9b69bbf4c6df"}, { new NonTerminator(NonTerminatorType.Variable), "bafc1f87-7778-4ac0-aa45-59d7d5226923"}, { new Terminator(DelimiterType.LeftParenthesis), "caa330bc-cc14-4679-bafb-47b37c5f4ae0"}, { Terminator.IdentifierTerminator, "42943c17-2772-4672-826f-ba44db9d32cd"}, { new Terminator(KeywordType.Not), "55e8b619-3bfd-4ce1-9a2d-5a0a60bc27c2"}, { new Terminator(OperatorType.Minus), "df993462-ad4d-456e-8079-06b377073a3e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8b35d723-e8cc-4aeb-b653-0a9bfd41bb7e") },
		{ "51e258a3-2e92-4139-b537-69d46b634fbc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "51e258a3-2e92-4139-b537-69d46b634fbc") },
		{ "d8e226c9-63ad-4ee2-9fc3-d34f2019eb62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d8e226c9-63ad-4ee2-9fc3-d34f2019eb62") },
		{ "823e144a-de85-4656-bd9d-9fa8da74fd5c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "823e144a-de85-4656-bd9d-9fa8da74fd5c") },
		{ "b6c55903-13af-4bf0-965f-27c62ebb3a54", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "cc731bc7-ea18-48bb-95b0-2cf22751f5e7"}, { new Terminator(OperatorType.Plus), "bee54548-de25-4e96-8c73-9e234b72bc44"}, { new Terminator(OperatorType.Minus), "d94f09ba-e20f-4b58-8e43-bad97eb903f4"}, { new Terminator(KeywordType.Or), "65027cc5-09b2-4c80-b933-3747f52730e9"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "b6c55903-13af-4bf0-965f-27c62ebb3a54") },
		{ "9e489b6e-aebf-4bf0-b95a-1e8b4c30d0e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "9c63650c-ac7e-453f-999f-3c32966c9e3a"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9e489b6e-aebf-4bf0-b95a-1e8b4c30d0e2") },
		{ "364a7211-dfcd-4426-985b-e5eef6b8b3d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "364a7211-dfcd-4426-985b-e5eef6b8b3d1") },
		{ "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8") },
		{ "23db53df-9747-4c31-a859-5da5e46b654a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "23db53df-9747-4c31-a859-5da5e46b654a") },
		{ "9653fd90-788b-4082-a9c5-5bf9d878771a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9e4ef0d0-a214-4da4-b0e8-1dbc254916c6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "acd89dcf-5a58-46a0-b16d-f4f1bf61d95d"}, { new NonTerminator(NonTerminatorType.Term), "c4cfbf2e-bfc7-4fa0-9534-3832086049a4"}, { new NonTerminator(NonTerminatorType.Factor), "15f7f03e-89b0-4b94-b82c-f780ab6d2668"}, { Terminator.NumberTerminator, "1e55cb78-8ebd-47b0-af5e-601c90cb3927"}, { new NonTerminator(NonTerminatorType.Variable), "aff1b54e-2140-48a5-ac62-65651706cd17"}, { new Terminator(DelimiterType.LeftParenthesis), "fe073a91-3867-45bb-b17b-793e15e3f602"}, { Terminator.IdentifierTerminator, "5a5ea736-2683-45c7-b635-07630a0dfd22"}, { new Terminator(KeywordType.Not), "b409d27b-d192-4a8e-bae2-7545bdfc8a65"}, { new Terminator(OperatorType.Minus), "d7d7d5de-0ee1-49c4-99c7-725ab4cefbec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9653fd90-788b-4082-a9c5-5bf9d878771a") },
		{ "d8a5a803-55a0-48d9-9e26-cfa20c699613", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "fbfb116a-8731-4a89-b958-70ca15b44bf6"}, { new NonTerminator(NonTerminatorType.IdVarPart), "bea118b6-2776-4b0c-99d3-c7c560ac11a8"}, { new Terminator(DelimiterType.LeftSquareBracket), "45635b9f-6ecd-413c-ab83-bb1110cc2096"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d8a5a803-55a0-48d9-9e26-cfa20c699613") },
		{ "be7d2ff7-c402-48b9-8aeb-1e78a5e56089", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b42526b6-9442-44a2-8bd4-7c842d5e3912"}, { Terminator.NumberTerminator, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8"}, { new NonTerminator(NonTerminatorType.Variable), "23db53df-9747-4c31-a859-5da5e46b654a"}, { new Terminator(DelimiterType.LeftParenthesis), "9653fd90-788b-4082-a9c5-5bf9d878771a"}, { Terminator.IdentifierTerminator, "d8a5a803-55a0-48d9-9e26-cfa20c699613"}, { new Terminator(KeywordType.Not), "be7d2ff7-c402-48b9-8aeb-1e78a5e56089"}, { new Terminator(OperatorType.Minus), "17272433-e4b7-42e0-9d3a-d5c5e24057cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "be7d2ff7-c402-48b9-8aeb-1e78a5e56089") },
		{ "17272433-e4b7-42e0-9d3a-d5c5e24057cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "bb6885d8-cece-4e77-a19d-6e6930179404"}, { Terminator.NumberTerminator, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8"}, { new NonTerminator(NonTerminatorType.Variable), "23db53df-9747-4c31-a859-5da5e46b654a"}, { new Terminator(DelimiterType.LeftParenthesis), "9653fd90-788b-4082-a9c5-5bf9d878771a"}, { Terminator.IdentifierTerminator, "d8a5a803-55a0-48d9-9e26-cfa20c699613"}, { new Terminator(KeywordType.Not), "be7d2ff7-c402-48b9-8aeb-1e78a5e56089"}, { new Terminator(OperatorType.Minus), "17272433-e4b7-42e0-9d3a-d5c5e24057cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "17272433-e4b7-42e0-9d3a-d5c5e24057cb") },
		{ "a581004e-ba9a-4061-9bf6-d2e738c290bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "eac9c4a3-bbed-49e9-83e4-742f9a804fdc"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "a581004e-ba9a-4061-9bf6-d2e738c290bf") },
		{ "d16fa4ca-95e9-465c-8022-3ac13b82a5b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "d16fa4ca-95e9-465c-8022-3ac13b82a5b4") },
		{ "a0903bba-0ae6-4034-baad-e95606db659c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a0903bba-0ae6-4034-baad-e95606db659c") },
		{ "d1d964d3-e97c-4fd0-828b-1fd0c920ea1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "982d46b9-a649-4594-9788-27e17aac2195"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d1d964d3-e97c-4fd0-828b-1fd0c920ea1a") },
		{ "ff5e7c0c-b3e5-4297-8526-321729adb384", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "e7c719ab-ee76-4c99-b8d0-b429977ce835"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ff5e7c0c-b3e5-4297-8526-321729adb384") },
		{ "ec2ad6f2-68db-4a08-9610-fb81f872429d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c2613bbd-7b4a-4ffc-910b-4669b7bc7b0d"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ec2ad6f2-68db-4a08-9610-fb81f872429d") },
		{ "e7075224-4686-443d-a873-8b465777ae51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "e7075224-4686-443d-a873-8b465777ae51") },
		{ "4f22a56b-267b-4196-9d9a-1118c90e7d45", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "4f22a56b-267b-4196-9d9a-1118c90e7d45") },
		{ "df953bf7-7d51-48d1-bd2c-115ddae453ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "2b531039-b7ed-45b3-b928-94e159450f62"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df953bf7-7d51-48d1-bd2c-115ddae453ef") },
		{ "977d195d-6a91-4247-be5d-8b30fe200b1b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "cbc77c7d-a297-42ad-a2b0-f56e0749b63c"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "977d195d-6a91-4247-be5d-8b30fe200b1b") },
		{ "451ea151-ba9e-45d8-8e79-c85a60d9b8e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "3259e142-002b-472c-b2a6-5a2be0940350"}, { new NonTerminator(NonTerminatorType.Factor), "26001bff-97ea-41b3-a426-f1a0b970d340"}, { Terminator.NumberTerminator, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47"}, { new NonTerminator(NonTerminatorType.Variable), "62195e40-a979-4524-9d19-3768c9af6073"}, { new Terminator(DelimiterType.LeftParenthesis), "20e7d417-07fc-4b80-a45d-01089114c58f"}, { Terminator.IdentifierTerminator, "fd68001e-0c67-4d09-820d-b22d33a6c89d"}, { new Terminator(KeywordType.Not), "2ef67070-7169-4318-8725-6117f46ed3fe"}, { new Terminator(OperatorType.Minus), "06cefe9b-a60e-4214-807e-2ddbf587c264"},}, new Dictionary<Terminator, ReduceInformation>{ }, "451ea151-ba9e-45d8-8e79-c85a60d9b8e0") },
		{ "c87c5ade-7537-424a-874d-ff8e2315da69", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "635534f1-9e77-411c-b951-55f71641c383"}, { Terminator.NumberTerminator, "41a7e5f1-f4b3-4304-8b4b-fbd0a097fc47"}, { new NonTerminator(NonTerminatorType.Variable), "62195e40-a979-4524-9d19-3768c9af6073"}, { new Terminator(DelimiterType.LeftParenthesis), "20e7d417-07fc-4b80-a45d-01089114c58f"}, { Terminator.IdentifierTerminator, "fd68001e-0c67-4d09-820d-b22d33a6c89d"}, { new Terminator(KeywordType.Not), "2ef67070-7169-4318-8725-6117f46ed3fe"}, { new Terminator(OperatorType.Minus), "06cefe9b-a60e-4214-807e-2ddbf587c264"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c87c5ade-7537-424a-874d-ff8e2315da69") },
		{ "b837453e-6734-4ed9-b83c-75008a9131c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "85d92e2d-299c-45f1-83a1-c6da3c8e4548"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b837453e-6734-4ed9-b83c-75008a9131c4") },
		{ "10791e8d-ebd0-4ffa-8418-3724332af02d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ca5ccf1a-4dfb-4793-aa13-29886cd6ca70"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "10791e8d-ebd0-4ffa-8418-3724332af02d") },
		{ "9ae20f4e-fb27-4c38-824b-c2c6ceb93582", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "9ae20f4e-fb27-4c38-824b-c2c6ceb93582") },
		{ "3823e6d1-d514-412d-8d64-77ee7e6c7541", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "f5c9af96-d17b-4951-b749-c38531200b75"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3823e6d1-d514-412d-8d64-77ee7e6c7541") },
		{ "655d58c6-903e-4eb2-a39a-083ad64393de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "655d58c6-903e-4eb2-a39a-083ad64393de") },
		{ "26c3389e-83b1-4edc-92f8-ff364328ec77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "26c3389e-83b1-4edc-92f8-ff364328ec77") },
		{ "751a3cbb-06d3-41fb-92c1-5783ed65f60f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "751a3cbb-06d3-41fb-92c1-5783ed65f60f") },
		{ "e5247481-e0a9-4897-9675-c22910c5c5b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e5247481-e0a9-4897-9675-c22910c5c5b5") },
		{ "36630a09-3be1-4c05-af14-281bc3a4ea29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "36630a09-3be1-4c05-af14-281bc3a4ea29") },
		{ "6339d066-ac25-4885-91d1-a857e19c4e0d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "c2d5ac2f-c08c-49c9-abbc-ab38f68bc383"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6339d066-ac25-4885-91d1-a857e19c4e0d") },
		{ "cc731bc7-ea18-48bb-95b0-2cf22751f5e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "fbf455ee-bd65-42ee-b892-a5b58a7c4f44"}, { new NonTerminator(NonTerminatorType.Factor), "364a7211-dfcd-4426-985b-e5eef6b8b3d1"}, { Terminator.NumberTerminator, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8"}, { new NonTerminator(NonTerminatorType.Variable), "23db53df-9747-4c31-a859-5da5e46b654a"}, { new Terminator(DelimiterType.LeftParenthesis), "9653fd90-788b-4082-a9c5-5bf9d878771a"}, { Terminator.IdentifierTerminator, "d8a5a803-55a0-48d9-9e26-cfa20c699613"}, { new Terminator(KeywordType.Not), "be7d2ff7-c402-48b9-8aeb-1e78a5e56089"}, { new Terminator(OperatorType.Minus), "17272433-e4b7-42e0-9d3a-d5c5e24057cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cc731bc7-ea18-48bb-95b0-2cf22751f5e7") },
		{ "9c63650c-ac7e-453f-999f-3c32966c9e3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9c6ae102-67a0-45e2-8ba7-1c37f97bf5d3"}, { Terminator.NumberTerminator, "b41f6a05-0d5e-46a4-8f7d-2c3cd5c5a6c8"}, { new NonTerminator(NonTerminatorType.Variable), "23db53df-9747-4c31-a859-5da5e46b654a"}, { new Terminator(DelimiterType.LeftParenthesis), "9653fd90-788b-4082-a9c5-5bf9d878771a"}, { Terminator.IdentifierTerminator, "d8a5a803-55a0-48d9-9e26-cfa20c699613"}, { new Terminator(KeywordType.Not), "be7d2ff7-c402-48b9-8aeb-1e78a5e56089"}, { new Terminator(OperatorType.Minus), "17272433-e4b7-42e0-9d3a-d5c5e24057cb"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c63650c-ac7e-453f-999f-3c32966c9e3a") },
		{ "9e4ef0d0-a214-4da4-b0e8-1dbc254916c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "d7684c67-866c-451f-ae60-cda712e15d15"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9e4ef0d0-a214-4da4-b0e8-1dbc254916c6") },
		{ "fbfb116a-8731-4a89-b958-70ca15b44bf6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "a9ab66ed-05e1-4abf-9d88-7d82f8804a5b"}, { new NonTerminator(NonTerminatorType.Expression), "8eec9a7f-4145-44bf-9197-18a5fe6d7f5f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "faf7f4cb-301e-45d9-9745-b6d82ce096b8"}, { new NonTerminator(NonTerminatorType.Term), "d87cf8df-734d-45bd-b403-027d25cee4b4"}, { new NonTerminator(NonTerminatorType.Factor), "d1496f83-a6e2-46ac-b6b4-8d84a52c00f4"}, { Terminator.NumberTerminator, "3324f6d0-11d5-4c03-b7cc-86d107c5c9ea"}, { new NonTerminator(NonTerminatorType.Variable), "d0593c1e-7097-4b1c-b2b3-340c0ce5f5c5"}, { new Terminator(DelimiterType.LeftParenthesis), "43def0a3-5d0a-4fe6-ac26-aeff86ab398a"}, { Terminator.IdentifierTerminator, "e9da9819-f2cf-4a67-9513-48874b3bbce2"}, { new Terminator(KeywordType.Not), "1af55271-632f-4aa4-b4ef-31530ba39a80"}, { new Terminator(OperatorType.Minus), "b5b15cf7-d149-4a37-8ac2-619e23d6076b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fbfb116a-8731-4a89-b958-70ca15b44bf6") },
		{ "bea118b6-2776-4b0c-99d3-c7c560ac11a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "bea118b6-2776-4b0c-99d3-c7c560ac11a8") },
		{ "45635b9f-6ecd-413c-ab83-bb1110cc2096", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "cd5edb22-8faa-4a6e-bed6-1d392078d5ae"}, { new NonTerminator(NonTerminatorType.Expression), "34aaf0d9-64b0-4678-a10c-a1e4e664fce6"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "1348560a-2744-469d-87a6-092c03873ff0"}, { new NonTerminator(NonTerminatorType.Term), "154bd632-5bfd-4450-9525-771afbbdf5dd"}, { new NonTerminator(NonTerminatorType.Factor), "4b7672cc-8831-48ba-b603-082dcc6c0aec"}, { Terminator.NumberTerminator, "e41ae64c-f16e-44bd-8e33-3b3a3264e9e7"}, { new NonTerminator(NonTerminatorType.Variable), "fafa494b-d3c6-4502-aef9-ad41dc354967"}, { new Terminator(DelimiterType.LeftParenthesis), "a4490596-3b7b-4793-b558-c732bcced8bd"}, { Terminator.IdentifierTerminator, "d3b52b7e-dbc2-4f30-ba1d-f8d1f0721708"}, { new Terminator(KeywordType.Not), "8c87e7ad-3592-46b9-8fa5-78261721f178"}, { new Terminator(OperatorType.Minus), "5eaad4b8-8722-4df1-bd50-173292b1abad"},}, new Dictionary<Terminator, ReduceInformation>{ }, "45635b9f-6ecd-413c-ab83-bb1110cc2096") },
		{ "b42526b6-9442-44a2-8bd4-7c842d5e3912", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "b42526b6-9442-44a2-8bd4-7c842d5e3912") },
		{ "bb6885d8-cece-4e77-a19d-6e6930179404", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "bb6885d8-cece-4e77-a19d-6e6930179404") },
		{ "982d46b9-a649-4594-9788-27e17aac2195", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "982d46b9-a649-4594-9788-27e17aac2195") },
		{ "e7c719ab-ee76-4c99-b8d0-b429977ce835", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "e7c719ab-ee76-4c99-b8d0-b429977ce835") },
		{ "2b531039-b7ed-45b3-b928-94e159450f62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "2b531039-b7ed-45b3-b928-94e159450f62") },
		{ "cbc77c7d-a297-42ad-a2b0-f56e0749b63c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "cbc77c7d-a297-42ad-a2b0-f56e0749b63c") },
		{ "3259e142-002b-472c-b2a6-5a2be0940350", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c87c5ade-7537-424a-874d-ff8e2315da69"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "3259e142-002b-472c-b2a6-5a2be0940350") },
		{ "635534f1-9e77-411c-b951-55f71641c383", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "635534f1-9e77-411c-b951-55f71641c383") },
		{ "85d92e2d-299c-45f1-83a1-c6da3c8e4548", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "85d92e2d-299c-45f1-83a1-c6da3c8e4548") },
		{ "ca5ccf1a-4dfb-4793-aa13-29886cd6ca70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9e70475a-5607-46e1-817e-6610d65f9579"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ca5ccf1a-4dfb-4793-aa13-29886cd6ca70") },
		{ "f5c9af96-d17b-4951-b749-c38531200b75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1744f434-5a13-4f39-bb35-0a633ef849c7"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f5c9af96-d17b-4951-b749-c38531200b75") },
		{ "c2d5ac2f-c08c-49c9-abbc-ab38f68bc383", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "22ebe382-ae78-4a15-84ce-2f879604ebd4"}, { new NonTerminator(NonTerminatorType.Variable), "5c9c6125-5edd-4aaf-a82f-9a1b582d4b36"}, { Terminator.IdentifierTerminator, "d74ee885-45f2-4595-a66a-53e932d8391f"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "f3e03d79-97a5-40ad-b6c5-8a90fcf0d2c0"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "b8794192-f4c9-4a1b-916f-bfaf49f43208"}, { new Terminator(KeywordType.If), "f6810f84-1a46-4894-8c44-c2f8068e5c2f"}, { new Terminator(KeywordType.For), "954e1e88-c806-45af-84bf-a00026c87a9c"}, { new Terminator(KeywordType.Begin), "52ecab85-fa9a-45b6-942d-b6de60d7087c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "c2d5ac2f-c08c-49c9-abbc-ab38f68bc383") },
		{ "fbf455ee-bd65-42ee-b892-a5b58a7c4f44", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "9c63650c-ac7e-453f-999f-3c32966c9e3a"}, { new Terminator(OperatorType.Multiply), "e0f8ce47-09b6-4bd5-86c7-e63662ad872b"}, { new Terminator(OperatorType.Divide), "00b1e30f-225b-4646-9de1-01d6655c252d"}, { new Terminator(KeywordType.Divide), "b75ade6d-fec8-4af0-a07c-4299e12e924f"}, { new Terminator(KeywordType.Mod), "e5f4e1f6-05cd-462c-8b33-5270c025dccd"}, { new Terminator(KeywordType.And), "942890b9-2bb6-47b5-8481-9efab246471d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "fbf455ee-bd65-42ee-b892-a5b58a7c4f44") },
		{ "9c6ae102-67a0-45e2-8ba7-1c37f97bf5d3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "9c6ae102-67a0-45e2-8ba7-1c37f97bf5d3") },
		{ "d7684c67-866c-451f-ae60-cda712e15d15", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "d7684c67-866c-451f-ae60-cda712e15d15") },
		{ "a9ab66ed-05e1-4abf-9d88-7d82f8804a5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "44a8204c-a1e7-4773-8bb7-bdf0f9b7d433"}, { new Terminator(DelimiterType.Comma), "49111152-c351-4b8f-9641-f66a8cc03a71"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a9ab66ed-05e1-4abf-9d88-7d82f8804a5b") },
		{ "cd5edb22-8faa-4a6e-bed6-1d392078d5ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "6a79e7a3-5d15-4f89-8350-4a68b85e03b8"}, { new Terminator(DelimiterType.Comma), "7c137af1-0a27-40a0-b473-5b4f68a14794"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cd5edb22-8faa-4a6e-bed6-1d392078d5ae") },
		{ "9e70475a-5607-46e1-817e-6610d65f9579", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "9e70475a-5607-46e1-817e-6610d65f9579") },
		{ "1744f434-5a13-4f39-bb35-0a633ef849c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1744f434-5a13-4f39-bb35-0a633ef849c7") },
		{ "22ebe382-ae78-4a15-84ce-2f879604ebd4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "22ebe382-ae78-4a15-84ce-2f879604ebd4") },
		{ "44a8204c-a1e7-4773-8bb7-bdf0f9b7d433", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "44a8204c-a1e7-4773-8bb7-bdf0f9b7d433") },
		{ "6a79e7a3-5d15-4f89-8350-4a68b85e03b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "6a79e7a3-5d15-4f89-8350-4a68b85e03b8") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public override ITransformer BeginTransformer => s_transformers["9bb88e74-e446-4ba5-a973-98aa20d700b1"];
	public override NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
